<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Lsd
 * @subpackage Lsd/admin
 * @author     ORION <support@designersuiteforwp.com>
 */

/**
 * Admin class
 */
class Lsd_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name       The name of this plugin.
     * @param      string $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        wp_enqueue_style($this->plugin_name, LSD_URL . 'admin/css/lsd-admin.css', array(), $this->version, 'all');
        if (is_lsd_admin_screen()) {
            wp_enqueue_style('lsd-simplegrid', LSD_URL . 'admin/css/simplegrid.min.css', array(), $this->version, 'all');
            wp_enqueue_style('lsd-colorpicker-css', LSD_URL . 'admin/js/colorpicker/css/colorpicker.css', array(), $this->version, 'all');
            //wp_enqueue_style('lsd-tooltip-css', LSD_URL . 'admin/css/tooltip.min.css', array(), $this->version, 'all');
            wp_enqueue_style('lsd-o-ui', LSD_URL . 'admin/css/UIs.css', array(), $this->version, 'all');
            wp_enqueue_style('lsd-select2-css', LSD_URL . 'admin/css/select2.min.css', array(), $this->version, 'all');
            wp_enqueue_style('o-lsd-flexgrid', LSD_URL . 'admin/css/flexiblegs.css', array(), $this->version, 'all');
            wp_enqueue_style('lsd-bs-modal-admin-css', LSD_URL . 'public/js/modal/modal.min.css', array(), $this->version, 'all');
        }
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        if (is_lsd_admin_screen()) {
            wp_enqueue_script('lsd-tabs-js', LSD_URL . 'admin/js/SpryTabbedPanels.min.js', array('jquery'), $this->version, false);
            // wp_enqueue_script('lsd-tooltip-js', LSD_URL . 'admin/js/tooltip.js', array('jquery'), $this->version, false);
            wp_enqueue_script('lsd-colorpicker-js', LSD_URL . 'admin/js/colorpicker/js/colorpicker.js', array('jquery'), $this->version, false);
            wp_enqueue_script('lsd-jquery-cookie-js', LSD_URL . 'admin/js/jquery.cookie.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script('o-admin', LSD_URL . 'admin/js/o-admin.js', array('jquery', 'jquery-ui-sortable'), $this->version, false);
            wp_localize_script('o-admin', 'home_url', home_url('/'));
            wp_enqueue_script('select2-js', LSD_URL . 'admin/js/select2.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script('lsd-bs-modal-admin-js', LSD_URL . 'public/js/modal/lsd-modal.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->plugin_name, LSD_URL . 'admin/js/lsd-admin.js', array('jquery', 'select2-js', 'lsd-bs-modal-admin-js'), $this->version, false);
            wp_localize_script($this->plugin_name, 'lsd_ajax_object', array('lsd_ajax_url' => admin_url('admin-ajax.php')));
            
            
            
            
        }
    }

    /**
     * Newsletter subscription
     */
    public function lsd_subscribe() {
        if (isset($_POST['email']) && wp_verify_nonce($_POST['lsd_subscription_nonce'], basename(__FILE__))) {// Input var okay.
            $email = sanitize_text_field(wp_unslash($_POST['email'])); // Input var okay.
            if (preg_match('#^[\w.-]+@[\w.-]+\.[a-z]{2,6}$#i', $email)) {
                $url = 'https://designersuiteforwp.com/service/osubscribe/v1/subscribe/?email=' . $email;
                $args = array('timeout' => 120);
                $response = wp_remote_get($url, $args);

                if (is_wp_error($response)) {
                    $error_message = $response->get_error_message();
                    echo esc_html__("Something went wrong: $error_message");
                    die();
                }
                if (isset($response['body'])) {
                    $answer = $response['body'];
                    if ('true' === $answer) {
                        update_option('o-lsd-subscribe', 'subscribed');
                        echo esc_html__($answer);
                    } else {
                        echo esc_html__($answer);
                    }

                    die();
                }
            } else {
                echo 'Please enter a valid email address';
                die();
            }
        } else {
            echo 'Nonce not verified.';
            die();
        }
    }

    /**
     * Displays the subscription notice
     */
    public function lsd_get_subscription_notice() {
        if (!get_option('o-lsd-subscribe') && get_transient('lsd-hide-notice') !== 'hide' && is_lsd_admin_screen()) {
            wp_nonce_field(basename(__FILE__), 'lsd_subscription_nonce');
            ?>
            <div id="lsd-subscription-notice" class="notice notice-info">

                <div id="lsd-plug-logo-text" >
                    <img id="plug-logo" style="height:50px; width: 50px"src="<?php echo esc_url(LSD_URL); ?>/admin/images/lsd.png">
                    <p> 
                        <strong>Lettering Signs Designer</strong>: <?php esc_html_e(' Sign up now to receive new releases notices and important bugs fixes directly', 'lsd'); ?>
                        <br> <?php esc_html_e('into your inbox!', 'lsd'); ?>
                    </p>

                </div>

                <div id="lsd-plug-sucribe-form">
                    <input type="email" id="o_user_email" name="usermail" placeholder="Your email here">
                    <img id="lsd-subscribe-loader" style="display:none;" src="<?php echo esc_url(LSD_URL); ?>/admin/images/loader.gif" >
                    <button id="lsd-subscribe" class="button button-primary"><?php esc_attr_e('Subscribe', 'lsd'); ?></button>
                    <a id="lsd-dismiss"><?php esc_attr_e('Not now', 'lsd'); ?></a>
                </div>
            </div>
            <?php
        }
        ?>
        <div id="lsd-subscription-success-notice" class="notice notice-info is-dismissible" style="display:none;">
            <img src="<?php echo esc_url(LSD_URL); ?>/admin/images/lsd.png">
            <div> <strong>Lettering Signs Designer</strong>: <?php esc_html_e('Thank you for subscribing! ', 'lsd'); ?></div>
        </div>
        <?php
    }

    /**
     * hide the notice for two weeks
     */
    public function lsd_hide_notice() {
        set_transient('lsd-hide-notice', 'hide', 2 * WEEK_IN_SECONDS);
        echo 'ok';
        die();
    }

    /**
     * Builds all the plugin menu and submenu
     */
    public function lsd_add_submenu() {
        $icon = LSD_URL . 'admin/images/lsd-dashicon.svg';
        add_menu_page('Lettering & Signs Designer', 'LSD', 'manage_product_terms', 'lsd-manage-fonts', array($this, 'lsd_get_fonts_page'), $icon);
        add_submenu_page('lsd-manage-fonts', esc_html__('Fonts', 'lsd'), esc_html__('Fonts', 'lsd'), 'manage_product_terms', 'lsd-manage-fonts', array($this, 'lsd_get_fonts_page'));
        add_submenu_page('lsd-manage-fonts', esc_html__('Configurations', 'lsd'), esc_html__('Configurations', 'lsd'), 'manage_product_terms', 'edit.php?post_type=lsd-config', false);
        if (class_exists('Ofb')) {
            add_submenu_page('lsd-manage-fonts', __('Forms', 'lsd'), __('Forms', 'lsd'), 'manage_product_terms', 'edit.php?post_type=ofb', false);
        }
        add_submenu_page('lsd-manage-fonts', esc_html__('Settings', 'lsd'), esc_html__('Settings', 'lsd'), 'manage_product_terms', 'lsd-manage-settings', array($this, 'lsd_get_settings_page'));
        add_submenu_page('lsd-manage-fonts', esc_html__('Request a feature', 'lsd'), esc_html__('Request a feature', 'lsd'), 'manage_product_terms', 'lsd-request-feature', array($this, 'lsd_get_request_page'));
        add_submenu_page('lsd-manage-fonts', esc_html__('Getting Started', 'lsd'), esc_html__('Getting Started', 'lsd'), 'manage_product_terms', 'lsd-about', array($this, "lsd_get_about_page"));
    }

    /**
     * Builds the fonts management page
     */
    public function lsd_get_fonts_page() {
        include_once LSD_DIR . '/includes/lsd-add-fonts.php';
        lsd_manage_fonts();
    }

    /**
     * redirect user to the request systeme.
     */
    public function lsd_get_request_page() {
        return false;
    }

    /**
     * Notice user about fonts mangement
     */
    public function lsd_get_help_notices() {
        $screen = get_current_screen();
        if (is_lsd_admin_screen() && strpos($screen->base, 'lsd-manage-fonts') !== false) {
            echo '<div class="lsd-info">
                   <p><b>Lettering & Signs Designer: </b>' . esc_html__('Learn more about fonts management', 'lsd') . ' <a class="button" href="https://designersuiteforwp.com/tutorials/woocommerce-product-designer-tutorials/how-to-manage-fonts/" target="_blank">' . esc_html__('here', 'lsd') . '</a></p>
                </div>';
        }
        /*
        if (!class_exists('Imagick') && is_lsd_admin_screen()) {
            if (is_lsd_admin_screen()) {
                ?>
                <div id="lsd-subscription-success-notice" class="notice notice-info">
                    <?php esc_html_e('The required imagick classes are not found.', 'lsd'); ?> 
                    <a target="<?php esc_attr_e('_blank', 'lsd'); ?>" href="<?php echo esc_url('http://php.net/manual/en/imagick.setup.php', 'lsd'); ?>"> <?php esc_html_e('Click here', 'lsd'); ?> </a>
                    <?php esc_html_e('to learn how to install it. ', 'lsd'); ?>
                </div>  
                <?php
            }
        }*/
    }

    /**
     * add new upload types
     */
    public function lsd_add_custom_mime_types($mimes) {
        return array_merge(
                $mimes, array(
            'svg' => 'image/svg+xml',
            'ttf' => 'application/x-font-ttf',
                )
        );
    }

    /**
     * add new upload types
     */
    public function lsd_get_max_input_vars_php_ini() {
        $total_max_normal = ini_get('max_input_vars');
        $msg = 'Your max input var is <strong>' . $total_max_normal . '</strong> but this page contains <strong>{nb}</strong> fields. You may experience a lost of data after saving. In order to fix this issue, please increase <strong>the max_input_vars</strong> value in your php.ini file.';
        ?>
        <script type="text/javascript">
            var o_max_input_vars = <?php echo esc_attr($total_max_normal); ?>;
            var o_max_input_msg = "<?php echo esc_attr__($msg, 'lsd'); ?>";
        </script>         
        <?php
    }

    /**
     * Builds the settings page
     */
    public function lsd_get_settings_page() {
        lsd_remove_transients();
        if (isset($_POST) && !empty($_POST) && wp_verify_nonce($_POST['lsd_settings_nonce'], basename(__FILE__))) {
            $this->lsd_save_tab_options();
            global $wp_rewrite;
            $wp_rewrite->flush_rules(false);
        }
        wp_enqueue_media();
        ?>
        <form method="POST">
            <div id="lsd-settings">
                <?php wp_nonce_field(basename(__FILE__), 'lsd_settings_nonce'); ?>
                <div class="wrap">
                    <h2><?php esc_attr_e('Lettering & signs designer Settings', 'lsd'); ?></h2>
                    <br>
                </div>
                <div id="TabbedPanels1" class="TabbedPanels">
                    <ul class="TabbedPanelsTabGroup ">
                        <li class="TabbedPanelsTab " tabindex="1"><span><?php esc_attr_e('General', 'lsd'); ?></span> </li>
                        <li class="TabbedPanelsTab" tabindex="2"><span><?php esc_attr_e('Output', 'lsd'); ?></span></li>
                        <li class="TabbedPanelsTab" tabindex="3"><span><?php esc_attr_e('License', 'lsd'); ?></span></li>
                    </ul>

                    <div class="TabbedPanelsContentGroup">

                        <div class="TabbedPanelsContent">
                            <div class='lsd-grid lsd-grid-pad'>
                                <?php $this->lsd_get_general_settings(); ?>							  
                            </div>
                        </div>

                        <div class="TabbedPanelsContent">
                            <div class="lsd-grid lsd-grid-pad">
                                <?php
                                $this->lsd_get_output_settings();
                                ?>
                            </div> 
                        </div>

                        <div class="TabbedPanelsContent">
                            <div class="lsd-grid lsd-grid-pad">
                                <?php
                                $this->lsd_get_license_settings();
                                ?>
                            </div> 
                        </div>

                    </div>
                </div>
            </div>
            <input type="submit" value="<?php esc_attr_e('Save', 'lsd'); ?>" class="button button-primary button-large mg-top-10-i">
        </form>
        <?php
    }

    /**
     * Save the settings
     */
    private function lsd_save_tab_options() {
        if (isset($_POST) && !empty($_POST) && wp_verify_nonce($_POST['lsd_settings_nonce'], basename(__FILE__))) {
            if (isset($_POST['lsd-general-options'])) {
                update_option('lsd-general-options', $_POST['lsd-general-options']);
            }

            if (isset($_POST['lsd-output-options'])) {
                update_option('lsd-output-options', $_POST['lsd-output-options']);
            }

            if (isset($_POST['lsd-license-options'])) {
                update_option('lsd-license-options', $_POST['lsd-license-options']);
            }

            $this->lsd_init_globals();
            ?>
            <div id="message" class="updated below-h2"><p><?php echo esc_attr_e('Settings successfully saved.', 'lsd'); ?></p></div>
            <?php
        }
    }

    /**
     * init the global $lsd_settings variable
     *
     * @return array Settings
     */
    public function lsd_init_globals() {
        global $lsd_settings;
        $lsd_settings['lsd-general-options'] = get_option('lsd-general-options');
        $lsd_settings['lsd-output-options'] = get_option('lsd-output-options');
        $lsd_settings['lsd-license'] = get_option('lsd-license');
    }

    /**
     * Builds the skin settings options
     */
    private function lsd_get_skins_settings() {
        $skin_begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-skin-container',
            'table' => 'options',
        );
        $skins_arr = apply_filters(
                'lsd_configuration_skins', array(
            'LSD_Default_Skin' => esc_html__('Default', 'lsd'),
                )
        );

        $skins = array(
            'title' => esc_html__('Skin', 'lsd'),
            'name' => 'lsd-ui-options[skin]',
            'type' => 'select',
            'options' => $skins_arr,
            'class' => 'chosen_select_nostd lsd-config-skin',
            'desc' => esc_html__('Editor look and feel.', 'lsd'),
        );

        $skin_end = array('type' => 'sectionend');
        $skin_settings = apply_filters(
                'lsd_skins_settings', array(
            $skin_begin,
            $skins,
            $skin_end,
                )
        );
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($skin_settings);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Builds the general settings options
     *
     * @return array Settings
     */
    private function lsd_get_general_settings() {
        $options = array();

        $general_options_begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-general-options',
            'table' => 'options',
            'title' => esc_html__('General Settings', 'lsd'),
        );

        $args = array(
            'post_type' => 'page',
            'nopaging' => true,
        );

        $customizer_page = array(
            'title' => esc_html__('Design Page', 'lsd'),
            'desc' => esc_html__('This setting allows the plugin to locate the page where customizations are made. Please note that this page can only be accessed by our plugin and should not appear in any menu.', 'lsd'),
            'name' => 'lsd-general-options[lsd_page_id]',
            'type' => 'post-type',
            'default' => '',
            'class' => 'chosen_select_nostd',
            'args' => $args,
        );

        $content_filter = array(
            'title' => esc_html__('Automatically append canvas to the design page.', 'lsd'),
            'name' => 'lsd-general-options[lsd-content-filter]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define whether or not you want to use a shortcode to display the the customizer in the selected page. If this setting is set to No, you will need to manually put the shortcode [lsd-editor] in the selected customization page.', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $customizer_cart_display = array(
            'title' => esc_html__('Parts position in cart', 'lsd'),
            'name' => 'lsd-general-options[lsd-parts-position-cart]',
            'default' => 'thumbnail',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to set where to show your customized products parts on the cart page', 'lsd'),
            'options' => array(
                'thumbnail' => esc_html__('Thumbnail column', 'lsd'),
                'name' => esc_html__('Name column', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $user_account_download_button = array(
            'title' => esc_html__('Download design from user account page', 'lsd'),
            'name' => 'lsd-general-options[lsd-user-account-download-btn]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the download button on user account page', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $send_attachments = array(
            'title' => esc_html__('Send the design as an attachment', 'lsd'),
            'name' => 'lsd-general-options[lsd-send-design-mail]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to send or not the design by mail after checkout', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $add_to_cart_action = array(
            'title' => esc_html__('Redirect after adding a custom design to the cart?', 'lsd'),
            'name' => 'lsd-general-options[lsd-redirect-after-cart]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define what to do after adding a design to the cart', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_design_buttons_cart_page = array(
            'title' => esc_html__('Hide design buttons on shop page', 'lsd'),
            'name' => 'lsd-general-options[lsd-hide-btn-shop-pages]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the design button on the shop page', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_cart_button_for_custom_products = array(
            'title' => esc_html__('Hide Add to cart button on single product page', 'lsd'),
            'name' => 'lsd-general-options[lsd-hide-cart-button]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define whether or not you want to hide the add to cart button for custom products on the single products page.', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        
        $hide_cart_button_shop_page = array(
            'title' => esc_html__('Hide Add to cart button on shop page', 'lsd'),
            'name' => 'lsd-general-options[lsd-hide-cart-shop-pages]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the Add to cart button for customizable products on the shop page', 'lsd'),
            'options' => array(
                '1' => esc_html__('Yes', 'lsd'),
                '0' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        
        $hide_preview_button = array(
            'title' => esc_html__('Hide preview button on designer page', 'lsd'),
            'name' => 'lsd-general-options[lsd-hide-preview-button]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the preview buttons on designer page', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        
        $hide_editor_footer_msg = array(
            'title' => esc_html__("Hide editor's footer notice ", 'lsd'),
            'name' => 'lsd-general-options[lsd-hide-editor-notice]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the notice at the bottom of the editor', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        
        
        $editor_footer_msg = array(
            'title' => esc_html__("Editor footer notice ", 'lsd'),
            'name' => 'lsd-general-options[lsd-editor-notice]',
            'default' => esc_html__("Please note: The surface color is for illustrative purposes only and will not be produced. Self-adhesive vinyl letters are individually cut and come pre-spaced with application tape applied.", 'lsd'),
            'type' => 'textarea',
            'desc' => esc_html__('The notice at the bottom of the editor', 'lsd'),
            'class' => 'chosen_select_nostd',
        );

        $general_options_end = array('type' => 'sectionend');

        array_push($options, $general_options_begin);
        array_push($options, $customizer_page);
        array_push($options, $content_filter);
        array_push($options, $customizer_cart_display);
        array_push($options, $user_account_download_button);
        array_push($options, $send_attachments);
        array_push($options, $add_to_cart_action);
        array_push($options, $hide_cart_button_for_custom_products);
        array_push($options, $hide_design_buttons_cart_page);
        array_push($options, $hide_cart_button_shop_page);
        array_push($options, $hide_preview_button);
        array_push($options, $hide_editor_footer_msg);
        array_push($options, $editor_footer_msg);
        array_push($options, $general_options_end);

        $options = apply_filters('lsd_general_options', $options);
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Builds the output settings options
     *
     * @return array Settings
     */
    private function lsd_get_output_settings() {
        $options = array();

        $cmyk_attr = array();
        if (!class_exists('Imagick')) {
            $cmyk_attr = array(
                'disabled' => '',
            );
        }
        $cmyk_profil = array(
            'title' => esc_html__('CMYK profile (when CMYK mode is enabled)', 'lsd'),
            'name' => 'lsd-output-options[lsd-cmyk-profil]',
            'type' => 'file',
            'desc' => esc_html__('This option allows you to set your own CMYK profile to use during the output conversion.', 'lsd') . '<br>' .
            esc_html__('Disabled if the Imagemagick extension is not installed and active.', 'lsd'),
            'custom_attributes' => array_merge($cmyk_attr, array('alt' => 'color profil')),
            'set' => esc_html__('Set profile', 'lsd'),
            'remove' => esc_html__('Remove profile', 'lsd'),
        );
        $design_composition = array(
            'title' => esc_html__('Design Composition', 'lsd'),
            'name' => 'lsd-output-options[design-composition]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to display or not design composition in the order ', 'lsd'),
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        $use_order_id = array(
            'title' => esc_html__('Use order ID as zip name', 'lsd'),
            'name' => 'lsd-output-options[use-order-id-as-zip-name]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('This option will name each output archive using the order ID.', 'lsd'),
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $output_options_begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-output-options',
            'title' => esc_html__('Output Settings', 'lsd'),
            'table' => 'options',
        );

        $output_options_end = array(
            'type' => 'sectionend',
            'id' => 'lsd-output-options',
        );

        array_push($options, $output_options_begin);
        //array_push($options, $cmyk_profil);
        array_push($options, $design_composition);
        array_push($options, $use_order_id);
        array_push($options, $output_options_end);

        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Builds the social networks settings options
     *
     * @return array Settings
     */
    private function lsd_get_license_settings() {
        $options = array();
        $license_begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-license',
            'title' => esc_html__('License Settings', 'lsd'),
            'table' => 'options',
        );

        $license_end = array(
            'type' => 'sectionend',
            'id' => 'lsd-license',
        );
        $license_key = array(
            'title' => esc_html__('License Key', 'lsd'),
            'desc' => __('The license key can be found in the order receipt email received after your purchase or <a href="https://designersuiteforwp.com/my-account/orders/" target="_blank">here</a>.', 'lsd'),
            'name' => 'lsd-license-options[license-key]',
            'type' => 'text',
            'default' => '',
        );

        array_push($options, $license_begin);
        array_push($options, $license_key);
        array_push($options, $license_end);

        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    //Afficher la notice d'activation de la license.
    function lsd_get_license_activation_notice() {
        $options = get_option('lsd-license-options');
        if (!isset($options['license-key']) || '' === $options['license-key'] || is_null($options['license-key'])) {
            $url = admin_url('admin.php?page=lsd-manage-settings');
            ?>
            <div class="notice notice-error">
                <p><?php _e('<strong>Lettering and Signs Designer</strong>: No license key provided. Please, set your license key in the "license" tab <a href="' . $url . '"> here</a>.', 'lsd'); ?></p>
                <p></p>
            </div>
            <?php
        }

        //if ($_SERVER['REMOTE_ADDR'] != '127.0.0.1' && $_SERVER['REMOTE_ADDR'] != '::1') {
            if (!get_option('lsd-license-key') && is_lsd_admin_screen()) {
                ?>
                <div class="notice notice-error">
                    <p><?php _e('<strong>Lettering and Signs Designer</strong>: You have not activated your license yet. Please activate it in order to get the plugin working.', 'lsd'); ?></p>
                    <a class="button" id="lsd-activate"><?php _e("Activate", "lsd"); ?></a><img style="display:none;" id="lsd-spinner" src="<?php echo LSD_URL . 'admin/images/spinner.gif'; ?> ">
                    <p></p>
                    <div id="lsd-license-message"></div>
                </div>
                <?php
            }
        //}
    }

//Fonction chargée d'activer la license.
    function lsd_activate_license() {
        $options = get_option('lsd-license-options');
        if (isset($options['license-key']) && '' !== $options['license-key']) {
            $license_key = $options['license-key'];
            $site_url = get_site_url();
            $code = $_POST['code'];
            $item_id = LSD_PLUGIN_ID;
            $url = "https://designersuiteforwp.com/service/olicenses/v1/license/?license-key=" . urlencode($license_key) . "&siteurl=" . urlencode($site_url) . "&id=" . $item_id . "&code=" . $code;
            $args = array('timeout' => 60);
            $response = wp_remote_get($url, $args);

            if (is_wp_error($response)) {
                $error_message = $response->get_error_message();
                echo "Something went wrong: $error_message";
                die();
            }
            if (isset($response["body"])) {
                $answer = $response["body"];
            }

            if (is_array(json_decode($answer, true))) {
                $data = json_decode($answer, true);
                $success = update_option('lsd-license-key', $data['key'] );
                echo "200";
            } else {
                echo $answer;
            }
        } else {
            $url = admin_url('admin.php?page=lsd-manage-settings');
            echo ("License key not found. Please, set your license key in the 'license' tab <a href=" . $url . ">here</a>. ");
        }

        die();
    }

    /**
     * Function to verify license validity
     */
    public function lsd_verify_validity() {
        $options = get_option('lsd-license-options');
        if (isset($options['license-key']) && $options['license-key'] != "") {
            $license_key = $options['license-key'];
            $site_url = get_site_url();
            $item_id = LSD_PLUGIN_ID;
            $url = "https://designersuiteforwp.com/service/olicenses/v1/checking/?license-key=" . urlencode($license_key) . "&siteurl=" . urlencode($site_url) . "&id=" . $item_id;
            $args = array('timeout' => 60);
            $response = wp_remote_get($url, $args);

            if (!is_wp_error($response)) {
                if (isset($response["body"]) && intval($response["body"]) != 200) {
                    delete_option("lsd-license-key");
                }
            }
        } else {
            if (get_option("lsd-license-key")) {
                delete_option("lsd-license-key");
            }
        }
    }

    /**
     * Build the about page
     */
    public function lsd_get_about_page() {
        ?>
        <div id='lsd-about-page'>
            <div class="wrap">
                <div id="features-wrap">
                    <h2 class="feature-h2"><?php _e('Getting Started', 'lsd'); ?></h2>
                    <div class="list-posts-content">
                        <div class="o-wrap o-features xl-gutter-8">
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo LSD_URL . "admin/images/features/" ?>custom-product.svg">
                                <h3 class="lsd-title"><?php _e('CREATE A CUSTOM PRODUCT', 'lsd'); ?></h3>
                                <p><?php _e('This tutorial is a step by step guide to teach you how to create a custom product on a woocommerce store using the Lettering and Signs Designer plugin.', 'lsd'); ?></p>
                                <a href="https://designersuiteforwp.com/lettering-and-signs-designer/tutorial/how-to-create-a-woocommerce-custom-sign-or-lettering-product/" class="button" target="_blank"><?php _e('Learn More', 'lsd'); ?></a>
                            </div>
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo LSD_URL . "admin/images/features/" ?>pricing-lsd-01.svg">
                                <h3 class="lsd-title"><?php _e('CREATE A CUSTOM PRICING STRATEGY', 'lsd'); ?></h3>
                                <p><?php _e('The goal of this tutorial is to show you how you can create a custom pricing strategy for your products using the lettering and signs plugin.', 'lsd'); ?></p>
                                <a href="https://designersuiteforwp.com/lettering-and-signs-designer/tutorial/how-to-setup-a-custom-pricing-strategy-for-the-lettering-and-signs-plugin-for-woocommerce/" class="button" target="_blank"><?php _e('Learn More', 'lsd'); ?></a>
                            </div>
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo LSD_URL . "admin/images/features/" ?>add-fonts.svg">
                                <h3 class="lsd-title"><?php _e('HOW TO MANAGE FONTS', 'lsd'); ?></h3>
                                <p><?php _e('Fonts are one of the most basic features in online product customization and yet one of the most critical ones. LSD does not only allow you to use google fonts.', 'lsd'); ?></p>
                                <a href="https://designersuiteforwp.com/lettering-and-signs-designer/tutorial/how-to-manage-fonts-in-the-lettering-and-signs-plugin-for-woocommerce/" class="button" target="_blank"><?php _e('Learn More', 'lsd'); ?></a>
                            </div>
                        </div>
                    </div>
                </div>

            </div>  
        </div>
        <?php
    }
    
    /**
     * Runs the new version check and upgrade process
     * @return \LSD_Updater
     */
    function lsd_get_updater() {
        do_action('lsd_before_init_updater');
        require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-lsd-updater.php' );
        $updater = new LSD_Updater();
        $updater->init();
        require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-lsd-updating-manager.php' );
        $updater->setUpdateManager(new LSD_Updating_Manager(LSD_VERSION, $updater->versionUrl(), LSD_MAIN_FILE));
        do_action('lsd_after_init_updater');
        return $updater;
    }

}
