<?php
/**
 * Class lsd config.
 *
 * @link       https://designersuiteforwp.com
 * @since      1.0.0
 *
 * @package    Lsd
 * @subpackage Lsd/public
 */

/**
 * The
 *
 * @package    Lsd
 * @subpackage Lsd/public
 * @author     ORION <support@designersuiteforwp.com>
 */
class LSD_Config {

    public $id;
    public $settings;

    public function __construct($config_id = false) {
        if ($config_id) {
            $this->id = $config_id;
            $this->settings = get_post_meta($config_id, "lsd-metas", true);
        }
    }

    /**
     * Register le custom post type
     */
    public function lsd_register_cpt_config() {

        $labels = array(
            'name' => _x('Configurations', 'lsd'),
            'singular_name' => _x('Configurations', 'lsd'),
            'add_new' => _x('New configuration', 'lsd'),
            'add_new_item' => _x('New configuration', 'lsd'),
            'edit_item' => _x('Edit configuration', 'lsd'),
            'new_item' => _x('New configuration', 'lsd'),
            'view_item' => _x('View configuration', 'lsd'),
            'not_found' => _x('No configuration found', 'lsd'),
            'not_found_in_trash' => _x('No configuration in the trash', 'lsd'),
            'menu_name' => _x('Product Designer', 'lsd'),
            'all_items' => _x('Configurations', 'lsd'),
        );

        $args = array(
            'labels' => $labels,
            'hierarchical' => false,
            'description' => 'Configurations',
            'supports' => array('title'),
            'public' => false,
            'show_ui' => true,
            'show_in_menu' => false,
            'show_in_nav_menus' => false,
            'publicly_queryable' => false,
            'exclude_from_search' => true,
            'has_archive' => false,
            'query_var' => false,
            'can_export' => true,
        );

        register_post_type('lsd-config', $args);
    }

    /**
     * Builds the cpt metaboxes
     */
    public function lsd_get_config_metabox() {

        $screens = array('lsd-config');

        foreach ($screens as $screen) {

            add_meta_box(
                    'lsd-metas-skin', esc_html__('Skin', 'lsd'), array($this, 'lsd_get_config_skin_metabox'), $screen
            );

            add_meta_box(
                    'lsd-metas-canvas', esc_html__('Canvas', 'lsd'), array($this, 'lsd_get_config_canvas_metabox'), $screen
            );

            add_meta_box(
                    'lsd-metas-text', esc_html__('Text', 'lsd'), array($this, 'lsd_get_config_text_metabox'), $screen
            );

            add_meta_box(
                    'lsd-metas-fonts', esc_html__('Fonts', 'lsd'), array($this, 'lsd_get_config_fonts_metabox'), $screen
            );

            add_meta_box(
                    'lsd-metas-output', esc_html__('Output settings', 'lsd'), array($this, 'lsd_get_config_output_metabox'), $screen
            );

            add_meta_box(
                    'lsd-metas-pricing-rules', esc_html__('Pricing rules', 'lsd'), array($this, 'lsd_get_config_pricing_metabox'), $screen
            );

            if (class_exists('Ofb')) {
                add_meta_box(
                        'lsd-metas-form-builder', __('Form Builder', 'lsd'), array($this, 'lsd_get_config_form_builder'), $screen
                );
            }
        }
    }

    public function lsd_config_order_metabox($order) {
        return array(
            'normal' => join(
                    ",", array(// Arrange here as you desire
                'lsd-metas-skin',
                'lsd-metas-canvas',
                'lsd-metas-text',
                'lsd-metas-fonts',
                'lsd-metas-output',
                'lsd-metas-pricing-rules',
                'lsd-metas-form-builder',
                'submitdiv'
                    )
            ),
        );
    }

    /**
     * Config skin settings
     */
    public function lsd_get_config_skin_metabox() {
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-skin-container',
        );
        $skins_arr = apply_filters("lsd_configuration_skins", array(
            "LSD_Default_Skin" => __("Default", "lsd")
        ));

        $skins = array(
            'title' => __('Skin', 'lsd'),
            'name' => 'lsd-metas[skin]',
            'type' => 'select',
            'options' => $skins_arr,
            'default' => 'LSD_Default_Skin',
            'desc' => __('Editor look and feel.', 'lsd'),
        );
        $end = array('type' => 'sectionend');
        $settings = array(
            $begin,
            $skins,
            $end,
        );
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($settings);
        echo wp_kses($raw_output, $allowed_tags);
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the canvas meta box
     */
    public function lsd_get_config_canvas_metabox() {
        wp_enqueue_media();
        wp_nonce_field(basename(__FILE__), 'lsd_config_nonce');
        $args = array('post_status' => 'publish');
        $pages = get_pages($args);
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $pages_arr = array();

        foreach ($pages as $page) {
            $pages_arr[$page->ID] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-canvas-settings',
        );

        $dimensions_type = array(
            'title' => esc_html__('Dimensions type', 'lsd'),
            'name' => 'lsd-metas[dimensions-type]',
            'default' => 'canvas-dimensions',
            'desc' => esc_html__('How you want the image dimensions to be calculated.', 'lsd'),
            'type' => 'radio',
            'options' => array(
                'font-size' => esc_html__('Let user choose the font size (canvas dimensions will be calculated using the text dimensions)', 'lsd'),
                'canvas-dimensions' => esc_html__('Define the canvas dimensions', 'lsd'),
            ),
            'row_class' => 'lsd-dimensions-type',
        );

        $text_min = array(
            'title' => esc_html__('Min font size', 'lsd'),
            'name' => 'lsd-metas[min-size]',
            'type' => 'number',
            'desc' => esc_html__('The minimum font size that the user can define.', 'lsd'),
            'default' => 12,
        );
        $text_max = array(
            'title' => esc_html__('Max font size', 'lsd'),
            'name' => 'lsd-metas[max-size]',
            'type' => 'number',
            'desc' => esc_html__('The maximum font size that the user can define.', 'lsd'),
            'default' => 60,
        );

        $text_dimensions_unit = array(
            'title' => esc_html__('Unit', 'lsd'),
            'name' => 'lsd-metas[font-dimensions-unit]',
            'type' => 'select',
            'options' => array(
                'pt' => esc_html__('Point', 'lsd'),
                'mm' => esc_html__('Millimeter', 'lsd'),
                'cm' => esc_html__('Centimeter', 'lsd'),
                'px' => esc_html__('Pixels', 'lsd'),
                'inch' => esc_html__('Inch', 'lsd'),
            ),
            'default' => 'mm',
        );

        $text_dimensions = array(
            'title' => esc_html__('Text dimensions limits', 'lsd'),
            'type' => 'groupedfields',
            'desc' => esc_attr('min and max values that customer can define as text font size. ', 'lsd'),
            'fields' => array($text_min, $text_max, $text_dimensions_unit),
            'row_class' => 'show-if-dimensions-font',
        );

        $max_width = array(
            'title' => esc_html__('Default width', 'lsd'),
            'name' => 'lsd-metas[canvas-w]',
            'type' => 'number',
            'desc' => esc_html__('in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'lsd'),
            'default' => 144,
        );
        $max_height = array(
            'title' => esc_html__(' Default height', 'lsd'),
            'name' => 'lsd-metas[canvas-h]',
            'type' => 'number',
            'desc' => esc_html__('in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'lsd'),
            'default' => 30,
        );

        $dimensions_unit = array(
            'title' => esc_html__('Unit', 'lsd'),
            'name' => 'lsd-metas[canvas-dimensions-unit]',
            'type' => 'select',
            'options' => array(
                'pt' => esc_html__('Point', 'lsd'),
                'mm' => esc_html__('Millimeter', 'lsd'),
                'cm' => esc_html__('Centimeter', 'lsd'),
                'px' => esc_html__('Pixels', 'lsd'),
                'in' => esc_html__('Inch', 'lsd'),
            ),
            'default' => 'mm',
        );

        $dimensions = array(
            'title' => esc_html__('Default product dimensions', 'lsd'),
            'type' => 'groupedfields',
            'desc' => __('Width and Height of the design area in the browser. <span style="color: red">It can be smaller than the expected output files in order to fit inside the website pages but need to be a scaled version of the expected output.</span>', 'lsd'),
            'fields' => array($max_width, $max_height, $dimensions_unit),
            'row_class' => 'show-if-dimensions-canvas',
        );

        $show_font_size = array(
            'title' => esc_html__('Show font size', 'lsd'),
            'name' => 'lsd-metas[show-fontsize]',
            'default' => 'yes',
            'desc' => esc_html__('Display the font size obtained for the text (this only work when the canvas dimensions type is selected.) ', 'lsd'),
            'type' => 'radio',
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'row_class' => 'show-if-dimensions-canvas',
        );

        $keep_ratio = array(
            'title' => esc_html__('Keep ratio', 'lsd'),
            'name' => 'lsd-metas[keep-ratio]',
            'default' => 'no',
            'desc' => esc_html__('Wether or not the customer can ajust his design dimensions by keeping the ratio.', 'lsd'),
            'type' => 'radio',
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'row_class' => 'show-if-dimensions-canvas',
        );

        $show_baseline = array(
            'title' => esc_html__('Show baseline', 'lsd'),
            'name' => 'lsd-metas[show-baseline]',
            'default' => 'yes',
            'desc' => esc_html__('Display the baseline height for the text (this only work when selecting the font-size dimensions type.) ', 'lsd'),
            'type' => 'radio',
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'row_class' => 'show-if-dimensions-font',
        );

        $bg_type = array(
            'title' => esc_html__('Background color selector', 'lsd'),
            'name' => 'lsd-metas[bg-type]',
            'default' => 'colors',
            'type' => 'radio',
            'desc' => esc_html__('How the customer will set the canvas background color.', 'lsd'),
            'options' => array(
                'colors' => esc_html__('Colors Palette', 'lsd'),
                'colorpicker' => esc_html__('Colors Picker', 'lsd'),
                'none' => esc_html__('None', 'lsd'),
            ),
            'row_class' => 'lsd-bg-type',
        );

        $price = array(
            'title' => esc_html__('Price', 'lsd'),
            'name' => 'price',
            'type' => 'number',
        );

        $hex_code = array(
            'title' => esc_html__('HEX color code', 'lsd'),
            'name' => 'hex',
            'type' => 'color',
        );

        $hex_name = array(
            'title' => esc_html__('HEX color name', 'lsd'),
            'name' => 'name',
            'type' => 'text',
        );

        $default_hex_code = array(
            'title' => esc_html__('Default color', 'lsd'),
            'name' => 'lsd-metas[default-hex-code]',
            'type' => 'color',
            'default' => '#000000',
            'row_class' => 'show-if-canvas-colorpicker',
        );

        $bg_colors = array(
            'title' => esc_html__('Palette', 'lsd'),
            'name' => 'lsd-metas[bg-colors-palette]',
            'type' => 'repeatable-fields',
            'fields' => array($hex_code, $price, $hex_name),
            'desc' => esc_html__('Background color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'lsd'),
            'class' => 'striped',
            'add_btn_label' => esc_html__('Add color', 'lsd'),
            'row_class' => 'show-if-canvas-colors',
        );

        $end = array('type' => 'sectionend');
        $settings = array(
            $begin,
            $dimensions_type,
            $text_dimensions,
            $dimensions,
            $show_font_size,
            $keep_ratio,
            $show_baseline,
            $bg_type,
            $bg_colors,
            $default_hex_code,
            $end,
        );
        $raw_output = o_admin_fields($settings);
        echo wp_kses($raw_output, $allowed_tags);
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the text meta box
     */
    public function lsd_get_config_text_metabox() {
        wp_enqueue_media();

        $args = array('post_status' => 'publish');
        $pages = get_pages($args);
        $pages_arr = array();

        foreach ($pages as $page) {
            $pages_arr[$page->ID] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-text-settings',
        );

        $enable_style = array(
            'title' => esc_html__('Enable text style', 'lsd'),
            'name' => 'lsd-metas[enable-style]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the text style functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'row_class' => 'lsd-enable-style',
        );

        $bg_type = array(
            'title' => esc_html__('Color Type', 'lsd'),
            'name' => 'lsd-metas[text-color-type]',
            'default' => 'colors',
            'type' => 'radio',
            'desc' => esc_html__('How the customer will set the text color.', 'lsd'),
            'options' => array(
                'colors' => esc_html__('Colors Palette', 'lsd'),
                'colorpicker' => esc_html__('Colors Picker', 'lsd'),
            ),
            'row_class' => 'lsd-text-color-type',
        );

        $price = array(
            'title' => esc_html__('Price', 'lsd'),
            'name' => 'price',
            'type' => 'number',
        );

        $hex_code = array(
            'title' => esc_html__('HEX color code', 'lsd'),
            'name' => 'hex',
            'type' => 'color',
        );

        $hex_name = array(
            'title' => esc_html__('HEX color name', 'lsd'),
            'name' => 'name',
            'type' => 'text',
        );

        $default_hex_code = array(
            'title' => esc_html__('HEX color code', 'lsd'),
            'name' => 'lsd-metas[default-text-color]',
            'type' => 'color',
            'default' => '#ffffff',
            'row_class' => 'show-if-text-colorpicker',
        );

        $bg_colors = array(
            'title' => esc_html__('Palette', 'lsd'),
            'name' => 'lsd-metas[text-colors-palette]',
            'type' => 'repeatable-fields',
            'fields' => array($hex_code, $price, $hex_name),
            'desc' => esc_html__('Text color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'lsd'),
            'class' => 'striped',
            'add_btn_label' => esc_html__('Add color', 'lsd'),
            'row_class' => 'show-if-text-colors',
        );


        //Shadow colors.

        $enable_shadow = array(
            'title' => esc_html__('Enable shadow functionality', 'lsd'),
            'name' => 'lsd-metas[enable-shadow]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the shadow functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'row_class' => 'lsd-enable-shadow',
        );

        $shadow_type = array(
            'title' => esc_html__('Shadow Color Type', 'lsd'),
            'name' => 'lsd-metas[shadow-color-type]',
            'default' => 'colors',
            'type' => 'radio',
            'desc' => esc_html__('How the customer will set the text shadow color.', 'lsd'),
            'options' => array(
                'colors' => esc_html__('Colors Palette', 'lsd'),
                'colorpicker' => esc_html__('Colors Picker', 'lsd'),
            ),
            'row_class' => 'lsd-shadow-type lsd-shadow-field',
        );

        $sh_price = array(
            'title' => esc_html__('Price', 'lsd'),
            'name' => 'price',
            'type' => 'number',
        );

        $sh_hex_code = array(
            'title' => esc_html__('HEX color code', 'lsd'),
            'name' => 'hex',
            'type' => 'color',
        );

        $sh_hex_name = array(
            'title' => esc_html__('HEX color name', 'lsd'),
            'name' => 'name',
            'type' => 'text',
        );

        $default_sh_hex_code = array(
            'title' => esc_html__('Default shadow color', 'lsd'),
            'name' => 'lsd-metas[default-shadow-color]',
            'value' => '#8a878a',
            'type' => 'color',
            'row_class' => 'show-if-shadow-colorpicker lsd-shadow-field',
        );

        $shadow_colors = array(
            'title' => esc_html__('Palette', 'lsd'),
            'name' => 'lsd-metas[shadow-colors-palette]',
            'type' => 'repeatable-fields',
            'fields' => array($sh_hex_code, $sh_price, $sh_hex_name),
            'desc' => esc_html__('Shadow color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'lsd'),
            'class' => 'striped',
            'add_btn_label' => esc_html__('Add color', 'lsd'),
            'row_class' => 'show-if-shadow-colors  lsd-shadow-field',
        );

        $shadow_max = array(
            'title' => esc_html__('Max distance', 'lsd'),
            'name' => 'lsd-metas[shadow-max]',
            'type' => 'number',
            'desc' => esc_html__('The maximum distance at which the text shadow can be placed.', 'lsd'),
            'default' => 10,
            'row_class' => 'lsd-shadow-field'
        );


        //Outline colors.

        $enable_outline = array(
            'title' => esc_html__('Enable outline functionality', 'lsd'),
            'name' => 'lsd-metas[enable-outline]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the outline functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'row_class' => 'lsd-enable-outline',
        );

        $outline_type = array(
            'title' => esc_html__('Outline Color Type', 'lsd'),
            'name' => 'lsd-metas[outline-color-type]',
            'default' => 'colors',
            'type' => 'radio',
            'desc' => esc_html__('How the customer will set the text outline color.', 'lsd'),
            'options' => array(
                'colors' => esc_html__('Colors Palette', 'lsd'),
                'colorpicker' => esc_html__('Colors Picker', 'lsd'),
            ),
            'row_class' => 'lsd-outline-type lsd-outline-field',
        );

        $out_price = array(
            'title' => esc_html__('Price', 'lsd'),
            'name' => 'price',
            'type' => 'number',
        );

        $out_hex_code = array(
            'title' => esc_html__('HEX color code', 'lsd'),
            'name' => 'hex',
            'type' => 'color',
        );

        $out_hex_name = array(
            'title' => esc_html__('HEX color name', 'lsd'),
            'name' => 'name',
            'type' => 'text',
        );

        $default_out_hex_code = array(
            'title' => esc_html__('Default outline color', 'lsd'),
            'name' => 'lsd-metas[default-outline-color]',
            'default' => '#8a878a',
            'type' => 'color',
            'row_class' => 'show-if-outline-colorpicker lsd-outline-field',
        );

        $outline_colors = array(
            'title' => esc_html__('Palette', 'lsd'),
            'name' => 'lsd-metas[outline-colors-palette]',
            'type' => 'repeatable-fields',
            'fields' => array($out_hex_code, $out_price, $out_hex_name),
            'desc' => esc_html__('Outline color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'lsd'),
            'class' => 'striped',
            'add_btn_label' => esc_html__('Add color', 'lsd'),
            'row_class' => 'show-if-outline-colors lsd-outline-field',
        );

        $outline_max = array(
            'title' => esc_html__('Max stroke', 'lsd'),
            'name' => 'lsd-metas[outline-max]',
            'type' => 'number',
            'desc' => esc_html__('The maximum stroke value for the text outline.', 'lsd'),
            'default' => 10,
            'row_class' => 'lsd-outline-field'
        );

        //Text alignement functionnality
        $enable_alignment = array(
            'title' => esc_html__('Enable text alignment', 'lsd'),
            'name' => 'lsd-metas[enable-alignment]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the text alignment functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
        );

        //Radius functionnality
        /*
          $enable_radius = array(
          'title' => esc_html__('Enable Curving', 'lsd'),
          'name' => 'lsd-metas[enable-curving]',
          'default' => 'no',
          'type' => 'radio',
          'desc' => esc_html__('By choosing "yes", you enable the curving functionality for you customers.', 'lsd'),
          'options' => array(
          'yes' => esc_html__('Yes', 'lsd'),
          'no' => esc_html__('No', 'lsd'),
          ),
          'row_class' => 'lsd-enable-radius'
          );

          $radius_max = array(
          'title' => esc_html__('Max radius', 'lsd'),
          'name' => 'lsd-metas[radius-max]',
          'type' => 'number',
          'desc' => esc_html__('The maximum radius angle.', 'lsd'),
          'default' => 360,
          'row_class' => 'lsd-radius-field'
          );
         */

        //Spacing functionnality
        $enable_spacing = array(
            'title' => esc_html__('Enable spacing', 'lsd'),
            'name' => 'lsd-metas[enable-spacing]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the spacing functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'row_class' => 'lsd-enable-spacing'
        );

        $spacing_max = array(
            'title' => esc_html__('Max spacing', 'lsd'),
            'name' => 'lsd-metas[spacing-max]',
            'type' => 'number',
            'desc' => esc_html__('The maximum spacing between text characters.', 'lsd'),
            'default' => 30,
            'row_class' => 'lsd-spacing-field'
        );

        //Text reverse functionnality.
        $enable_reverse = array(
            'title' => esc_html__('Enable reverse', 'lsd'),
            'name' => 'lsd-metas[enable-reverse]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('By choosing "yes", you enable the reverse functionality for you customers.', 'lsd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
        );

        $end = array('type' => 'sectionend');
        $settings = array(
            $begin,
            $bg_type,
            $bg_colors,
            $default_hex_code,
            $enable_style,
            $enable_shadow,
            $shadow_max,
            $shadow_type,
            $shadow_colors,
            $default_sh_hex_code,
            $enable_outline,
            $outline_max,
            $outline_type,
            $outline_colors,
            $default_out_hex_code,
            $enable_alignment,
            $enable_spacing,
            $spacing_max,
            $enable_reverse,
            //$enable_radius,
            //$radius_max,
            $end,
        );
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($settings);
        echo wp_kses($raw_output, $allowed_tags);
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the fonts meta box
     */
    public function lsd_get_config_fonts_metabox() {
        wp_enqueue_media();

        $args = array('post_status' => 'publish');
        $pages = get_pages($args);
        $pages_arr = array();

        foreach ($pages as $page) {
            $pages_arr[$page->ID] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-fonts-settings',
        );

        $use_global_fonts = array(
            'title' => esc_html__('Use global fonts', 'lsd'),
            'name' => 'lsd-metas[global-fonts]',
            'default' => 'yes',
            'desc' => esc_html__('Available fonts to use when designing.', 'lsd'),
            'type' => 'radio',
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'row_class' => 'lsd-fonts-type',
        );

        $lsd_fonts = lsd_refactor_fonts(get_option('lsd-fonts'));
        $chosen_fonts = array(
            'title' => esc_html__('Selected fonts', 'lsd'),
            'name' => 'lsd-metas[selected-fonts]',
            'desc' => esc_html__('Please select the fonts that should be available for use.', 'lsd'),
            'type' => 'select',
            'options' => $lsd_fonts,
            'row_class' => 'show-if-fonts-global-no',
            'id' => 'lsd_fonts_selector',
            'custom_attributes' => array(
                'multiple' => 'multiple',
            ),
        );

        $end = array('type' => 'sectionend');
        $settings = array(
            $begin,
            $use_global_fonts,
            $chosen_fonts,
            $end,
        );
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($settings);
        echo wp_kses($raw_output, $allowed_tags);
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the pricing metabox
     */
    public function lsd_get_config_pricing_metabox() {
        $post_id = get_the_ID();
        $lsd_metas = get_post_meta($post_id, 'lsd-metas', true);
        $allowed_tags = lsd_allowed_tags();

        $params = array(
            'if_canvas_square_unit' => esc_html__('If Canvas square unit (WxH)', 'lsd'),
            'txt_nb_chars' => esc_html__('NB chars in text', 'lsd'),
            'txt_nb_lines' => esc_html__('NB lines in text', 'lsd'),
            'if_selected_font' => esc_html__('If selected font', 'lsd'),
            'if_char_spacing' => esc_html__('If Character spacing', 'lsd'),
        );
        $first_rule = $this->lsd_get_rule_tpl($params, true);

        $rule_tpl = $this->lsd_get_rule_tpl($params, false);
        ?>
        <div class='lsd-rules-table-container'>
            <textarea id='lsd-rule-tpl' style='display: none;'>
                <?php echo esc_textarea($rule_tpl); ?>
            </textarea>
            <textarea id='lsd-first-rule-tpl' style='display: none;'>
                <?php echo esc_textarea($first_rule); ?>
            </textarea>
            <?php
            $pricing_rules = array();
            if (isset($lsd_metas['pricing-rules'])) {
                $pricing_rules = $lsd_metas['pricing-rules'];
            }
            if (is_array($pricing_rules) && !empty($pricing_rules)) {
                $rule_group = 0;
                foreach ($pricing_rules as $rules_group) {
                    $rule_index = 0;
                    $rules = $rules_group['rules'];
                    $a_price = $rules_group['a_price'];
                    if (isset($rules_group["scope"])) {
                        $scope = $rules_group["scope"];
                    } else {
                        $scope = 'item';
                    }
                    ?>
                    <table class="lsd-rules-table widefat">
                        <tbody>
                            <?php
                            foreach ($rules as $rule_arr) {
                                if (0 === $rule_index) {
                                    $rule_html = $this->lsd_get_rule_tpl($params, true, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value'], $a_price, $scope, count($rules));
                                } else {
                                    $rule_html = $this->lsd_get_rule_tpl($params, false, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value']);
                                }
                                $r1 = str_replace('{rule-group}', $rule_group, $rule_html);
                                $r2 = str_replace('{rule-index}', $rule_index, $r1);
                                //echo wp_kses($r2, $allowed_tags );
                                echo $r2;
                                $rule_index++;
                            }
                            $rule_group++;
                            ?>
                        </tbody>
                    </table>
                    <?php
                }
            } else {
                ?>
                <table class="lsd-rules-table widefat">
                    <tbody>
                        <?php
                        $rule_group = 0;
                        $rule_index = 0;
                        $r1 = str_replace('{rule-group}', $rule_group, $first_rule);
                        $r2 = str_replace('{rule-index}', $rule_index, $r1);
                        //Escaping output.
                        echo $r2;
                        ?>
                    </tbody>
                </table>
                <?php
            }
            ?>

        </div>
        <a class="button lsd-add-group mg-bot-10i">Add rule group</a>
        <?php
    }

    public function lsd_get_config_form_builder() {
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-form-builder-container',
        );

        $args = array(
            "post_type" => "ofb",
            "nopaging" => true,
        );

        $forms = array(
            'title' => __('Form Builder', 'wpd'),
            'desc' => __('The selected form will be displayed on the customization page to gather additional informations from the customer.', 'wpd'),
            'name' => "lsd-metas[form-builder]",
            'type' => 'post-type',
            'default' => 'default',
            'args' => $args
        );

        $end = array(
            'type' => 'sectionend',
        );

        $settings = array(
            $begin,
            $forms,
            $end,
        );
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($settings);
        echo wp_kses($raw_output, $allowed_tags);
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the output metabox
     */
    public function lsd_get_config_output_metabox() {
        global $lsd_settings;
        $output_global_settings = get_proper_value($lsd_settings, 'lsd-output-options', array());

        $options = array();
        $output_format = array(
            'title' => esc_html__('Output Format', 'lsd'),
            'name' => 'lsd-metas[output-settings][output-format]',
            'type' => 'radio',
            'row_class' => 'config-output-format',
            'options' => array(
                'png' => esc_html__('PNG', 'lsd'),
                'jpg' => esc_html__('JPG', 'lsd'),
                //'svg'     => esc_html__( 'SVG', 'lsd' ),
                'pdf+png' => esc_html__('PDF + PNG', 'lsd'),
                'pdf+jpg' => esc_html__('PDF + JPG', 'lsd'),
            //'pdf+svg' => esc_html__( 'PDF + SVG', 'lsd' ),
            ),
            'default' => 'png',
        );
        $output_loop_delay = array(
            'title' => esc_html__('Output loop delay (milliseconds)', 'lsd'),
            'name' => 'lsd-metas[output-settings][lsd-output-loop-delay]',
            'type' => 'text',
            'default' => get_proper_value($output_global_settings, 'lsd-output-loop-delay'),
        );

        $zip_output = array(
            'title' => esc_html__('Zip output file', 'lsd'),
            'name' => 'lsd-metas[output-settings][zip-output]',
            'type' => 'radio',
            'row_class' => 'zip-output',
            'options' => array(
                'yes' => esc_html__('Yes', 'lsd'),
                'no' => esc_html__('No', 'lsd'),
            ),
            'default' => 'no',
        );
        $zip_folder_name = array(
            'title' => esc_html__('Zip output folder prefix', 'lsd'),
            'name' => 'lsd-metas[output-settings][zip-folder-name]',
            'type' => 'text',
            'row_class' => 'show-if-zip',
        );
        $cmyk_conversion = array(
            'title' => esc_html__('CMYK conversion (Requires ImageMagick)', 'lsd'),
            'name' => 'lsd-metas[output-settings][lsd-cmyk-conversion]',
            'type' => 'radio',
            'row_class' => 'show-if-jpg',
            'options' => array(
                'no' => esc_html__('No', 'lsd'),
                'yes' => esc_html__('Yes', 'lsd'),
            ),
            'default' => get_proper_value($output_global_settings, 'lsd-cmyk-conversion'),
        );

        $pdf_format = array(
            'title' => esc_html__('PDF Format', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-format]',
            'type' => 'groupedselect',
            'desc' => esc_html__("Output PDF format and default dimensions ( used if the generated design's dimensions are smaller or equals to the default ones.)", 'lsd'),
            'data-id' => '',
            'row_class' => 'show-if-pdf',
            'options' => get_lsd_pdf_formats(),
            'default' => get_proper_value($output_global_settings, 'pdf-format'),
        );

        $pdf_w = array(
            'title' => esc_html__('Width', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-width]',
            'type' => 'number',
            'default' => get_proper_value($output_global_settings, 'pdf-width'),
        );

        $pdf_h = array(
            'title' => esc_html__('Height', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-height]',
            'type' => 'number',
            'default' => get_proper_value($output_global_settings, 'pdf-height'),
        );

        /*
          $pdf_unit = array(
          'title'   => esc_html__( 'Unit', 'lsd' ),
          'name'    => 'lsd-metas[output-settings][pdf-unit]',
          'type'    => 'select',
          'class'   => 'pdf-unit',
          'options' => array(
          'pt' => esc_html__( 'Point', 'lsd' ),
          'mm' => esc_html__( 'Millimeter', 'lsd' ),
          'cm' => esc_html__( 'Centimeter', 'lsd' ),
          'px' => esc_html__( 'Pixels', 'lsd' ),
          'in' => esc_html__( 'Inch', 'lsd' ),
          ),
          'default' => get_proper_value( $output_global_settings, 'pdf-unit' ),
          ); */

        $pdf_custom_dimensions = array(
            'title' => 'Custom PDF dimensions',
            'desc' => esc_html__("These dimensions will only be used if the PDF format is set to Custom.", 'lsd'),
            'type' => 'groupedfields',
            'row_class' => 'show-if-pdf',
            'fields' => array($pdf_w, $pdf_h, /* $pdf_unit */),
        );
        $pdf_resolution = array(
            'title' => 'PDF resolution (dpi)',
            'desc' => esc_html__('Output PDF resolution.', 'lsd'),
            'type' => 'number',
            'name' => 'lsd-metas[output-settings][resolution]',
            'row_class' => 'hide-if-pixels show-if-pdf',
            'default' => 300,
        );

        $pdf_margin_top_bottom = array(
            'title' => esc_html__('PDF Margin Top & Bottom', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-margin-tb]',
            'type' => 'text',
            'default' => get_proper_value($output_global_settings, 'pdf-margin-tb', 20),
        );

        $pdf_margin_left_right = array(
            'title' => esc_html__('PDF Margin Left & Right', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-margin-lr]',
            'type' => 'text',
            'default' => get_proper_value($output_global_settings, 'pdf-margin-lr', 20),
        );

        $margins = array(
            'title' => esc_html__('Margins', 'lsd'),
            'type' => 'groupedfields',
            'desc' => esc_html__('Vertical and Horizontal margins in the PDF file.', 'lsd'),
            'row_class' => 'show-if-pdf',
            'fields' => array($pdf_margin_top_bottom, $pdf_margin_left_right),
        );

        $crop_x = array(
            'title' => esc_html__('Center Abcissa', 'lsd'),
            'name' => 'x',
            'type' => 'number',
        );
        $crop_y = array(
            'title' => esc_html__('Center Ordinate', 'lsd'),
            'name' => 'y',
            'type' => 'number',
        );
        $crop_w = array(
            'title' => esc_html__('Width', 'lsd'),
            'name' => 'w',
            'type' => 'number',
        );
        $crop_h = array(
            'title' => esc_html__('Height', 'lsd'),
            'name' => 'h',
            'type' => 'number',
        );
        $type = array(
            'title' => esc_html__('Type', 'lsd'),
            'name' => 'type',
            'type' => 'text',
        );

        $crop_marks = array(
            'title' => esc_html__('Crop marks', 'lsd'),
            'name' => 'lsd-metas[output-settings][crop-marks]',
            'desc' => __('Crop marks settings.<br> <strong>Type</strong>: One symbol per type separated by comma: <ul><li>T = TOP</li><li>F = BOTTOM</li> <li>L = LEFT</li> <li>R = RIGHT</li><li>TL = A = TOP-LEFT</li><li>TR = B = TOP-RIGHT</li><li>BL = C = BOTTOM-LEFT</li><li>BR = D = BOTTOM-RIGHT</li></ul>', 'lsd'),
            'type' => 'repeatable-fields',
            'fields' => array($crop_x, $crop_y, $crop_w, $crop_h, $type),
            'row_class' => 'hide-if-pixels show-if-pdf',
            'ignore_desc_col' => false,
            'add_btn_label' => esc_html__('Add Mark', 'lsd'),
        );

        $bleed_x = array(
            'title' => esc_html__('H. spacing', 'lsd'),
            'name' => 'h',
            'type' => 'number',
            'custom_attributes' => array('step' => '0.01'),
        );
        $bleed_y = array(
            'title' => esc_html__('V. spacing', 'lsd'),
            'name' => 'v',
            'type' => 'number',
            'custom_attributes' => array('step' => '0.01'),
        );

        $bleed_color = array(
            'title' => esc_html__('Border color', 'lsd'),
            'name' => 'color',
            'type' => 'text',
            'custom_attributes' => array('placeholder' => '#FF0000 for ex.'),
        );

        $bleed_dash_spacing = array(
            'title' => esc_html__('Dash spacing', 'lsd'),
            'name' => 'dash-spacing',
            'type' => 'number',
            'custom_attributes' => array('step' => '0.01'),
        );

        $bleed_border_width = array(
            'title' => esc_html__('Border width', 'lsd'),
            'name' => 'border-width',
            'type' => 'number',
            'custom_attributes' => array('step' => '0.01'),
        );

        $bleed = array(
            'title' => __('Bleed, Trim & Safe zones', 'lsd'),
            'name' => 'lsd-metas[output-settings][zones]',
            'desc' => __('<ul><li><strong>H. spacing</strong>: Space between the left and right borders of the document and the area.</li><li><strong>V. spacing</strong>: Space between the top and bottom borders of the document and the area.</li></ul>', 'lsd'),
            'type' => 'repeatable-fields',
            'fields' => array($bleed_x, $bleed_y, $bleed_color, $bleed_dash_spacing, $bleed_border_width),
            'row_class' => 'hide-if-pixels show-if-pdf',
            'ignore_desc_col' => false,
            'add_btn_label' => __('Add zone', 'lsd'),
        );

        $pdf_orientation = array(
            'title' => __('PDF Orientation', 'lsd'),
            'name' => 'lsd-metas[output-settings][pdf-orientation]',
            'default' => 'P',
            'type' => 'select',
            'row_class' => 'show-if-pdf',
            'options' => array(
                'P' => esc_html__('Portrait', 'lsd'),
                'L' => esc_html__('Landscape', 'lsd'),
            ),
            'default' => get_proper_value($output_global_settings, 'pdf-orientation'),
        );

        $output_options_begin = array(
            'type' => 'sectionbegin',
            'title' => esc_html__('Output Settings', 'lsd'),
            'table' => 'metas',
            'id' => 'lsd_product_output',
        );

        $output_options_end = array(
            'type' => 'sectionend',
            'id' => 'lsd_product_output',
        );

        array_push($options, $output_options_begin);
        array_push($options, $output_format);
        //array_push( $options, $output_loop_delay );
        //array_push( $options, $cmyk_conversion );
        array_push($options, $pdf_format);
        array_push($options, $pdf_custom_dimensions);
        array_push($options, $margins);
        //array_push( $options, $crop_marks );
        //array_push( $options, $bleed );
        //array_push( $options, $pdf_resolution );
        array_push($options, $pdf_orientation);
        array_push($options, $zip_output);
        array_push($options, $zip_folder_name);
        array_push($options, $output_options_end);
        ?>

        <?php
        //Escaping output.
        $raw_output = o_admin_fields($options);
        echo $raw_output;
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl = <?php echo wp_json_encode($o_row_templates); ?>;
        </script>
        <?php
    }

    /**
     * Builds the pricing rules table
     *
     * @param type $params Array of parameters.
     * @param type $with_price Use with price.
     * @param type $default_param Default parameter.
     * @param type $default_operator Default operator.
     * @param type $default_value Default value.
     * @param type $default_price Default price.
     * @param type $default_scope Default scope.
     * @param type $count Count.
     * @return type
     */
    private function lsd_get_rule_tpl($params, $with_price = false, $default_param = false, $default_operator = '<', $default_value = '', $default_price = '', $default_scope = "per_item", $count = 1) {
        ob_start();
        $operators = array(
            '<' => esc_html__('is less than', 'lsd'),
            '<=' => esc_html__('is less or equal to', 'lsd'),
            '==' => esc_html__('equals', 'lsd'),
            '>' => esc_html__('more than', 'lsd'),
            '>=' => esc_html__('more or equal to', 'lsd'),
            '!=' => esc_html__('not equals', 'lsd'),
        );
        $scopes = array(
            "item" => esc_html__("Per item", "wpd"),
            "design" => esc_html__("On whole design", "wpd")
        );
        $allowed_tags = lsd_allowed_tags();
        ?>
        <tr data-id="rule_{rule-group}">
            <td class="param">
                <select id="lsd-group_{rule-group}_rule_{rule-index}_param" class="select lsd-pricing-group-param" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][param]">
                    <?php
                    foreach ($params as $param_key => $param_val) {
                        if ($param_key === $default_param) {
                            ?>
                            <option value='<?php echo esc_attr($param_key); ?>' selected="selected"><?php echo esc_attr($param_val); ?></option>
                            <?php
                        } else {
                            ?>
                            <option value='<?php echo esc_attr($param_key); ?>'><?php echo esc_attr($param_val); ?></option>
                            <?php
                        }
                    }
                    ?>
                </select>
            </td>
            <td class="operator">
                <select id="lsd-pricing-group_{rule-group}_rule_{rule-index}_operator" class="select" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][operator]">
                    <?php
                    $option = 0;
                    foreach ($operators as $operator_key => $operator_val) {
                        if ($operator_key === $default_operator) {
                            $op_html = "<option id='lsd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities($operator_key) . "' selected='selected'>" . $operator_val . '</option>';
                        } else {
                            $op_html = "<option id='lsd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities($operator_key) . "'>" . $operator_val . '</option>';
                        }
                        $op = str_replace('{option}', $option, $op_html);
                        echo wp_kses($op, $allowed_tags);
                        $option++;
                    }
                    ?>
                    <script type="text/javascript">
                        var operators =<?php echo wp_json_encode($operators); ?>;
                    </script>

                </select>
            </td>
            <td class="value">
                <input type="text" name="lsd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][value]" value="<?php echo esc_attr($default_value); ?>" placeholder="number">
            </td>
            <?php
            if ($with_price) {
                ?>
                <td class="a_price" rowspan="<?php echo esc_attr($count); ?>">
                    <input type="number" name="lsd-metas[pricing-rules][group_{rule-group}][a_price]" value="<?php echo esc_attr($default_price); ?>" placeholder="price" step="any">
                    <select id="lsd-pricing-group_{rule-group}_rule_{rule-index}_scope" class="select" name="lsd-metas[pricing-rules][group_{rule-group}][scope]">
                        <?php
                        foreach ($scopes as $scope_key => $scope_val) {
                            if ($scope_key == $default_scope) {
                                ?><option value='<?php echo $scope_key; ?>' selected="selected"><?php echo $scope_val; ?></option><?php
                            } else {
                                ?><option value='<?php echo $scope_key; ?>'><?php echo $scope_val; ?></option><?php
                            }
                        }
                        ?>
                    </select>
                </td>
                <?php
            }
            ?>
            <td class="add">
                <a class="lsd-add-rule button" data-group='{rule-group}'><?php echo esc_attr__('and', 'lsd'); ?></a>
            </td>
            <td class="remove">
                <a class="lsd-remove-rule acf-button-remove"></a>
            </td>
        </tr>
        <?php
        $rule_tpl = ob_get_contents();
        ob_end_clean();
        return $rule_tpl;
    }

    /**
     * Save the config
     *
     * @param type $post_id Post id.
     */
    public function lsd_save_config($post_id) {
        $meta_key = 'lsd-metas';
        $checkboxes = array('can-design-from-blank');
        if (isset($_POST[$meta_key]) && wp_verify_nonce($_POST['lsd_config_nonce'], basename(__FILE__))) {// Input var okay.
            $old_metas = get_post_meta($post_id, $meta_key, true);
            if (empty($old_metas)) {
                $old_metas = array();
            }
            //$new_metas = array_replace($old_metas, $_POST[$meta_key]); // Input var okay.
            $new_metas = $_POST[$meta_key]; // Input var okay.
            var_dump($new_metas["selected-fonts"]);
            // die();
            foreach ($checkboxes as $checkboxes_key) {
                if (!isset($_POST[$meta_key][$checkboxes_key]) && isset($new_metas[$checkboxes_key])) {// Input var okay.
                    unset($new_metas[$checkboxes_key]);
                }
            }
            update_post_meta($post_id, $meta_key, $new_metas);
        }
    }

    /**
     * Re order the metaboxes
     *
     * @param array $order Array of ordered elements.
     * @return string
     */
    public function lsd_get_metabox_order($order) {
        $order['advanced'] = 'lsd-metas-canvas,lsd-metas-text,lsd-metas-fonts,lsd-metas-pricing-rules,lsd-metas-output,lsd-metas-form-builder,submitdiv';
        return $order;
    }

    /**
     * Duplicate the config link
     *
     * @param string $actions Actions.
     * @param type   $post Post object.
     * @return string
     */
    public function lsd_duplicate_post_link($actions, $post) {
        if ('lsd-config' === $post->post_type && current_user_can('edit_posts')) {
            $actions['duplicate'] = '<a href="admin.php?action=lsd_duplicate_config&amp;post=' . $post->ID . '&amp;duplicate_nonce=' . wp_create_nonce(basename(__FILE__)) . '" title="Duplicate this item" rel="permalink">Duplicate</a>';
        }
        return $actions;
    }

    /**
     * Duplicate the configs
     */
    public function lsd_duplicate_config() {
        global $lsdb;

        /*
         * Nonce verification
         */
        if (!isset($_GET['duplicate_nonce']) || !wp_verify_nonce($_GET['duplicate_nonce'], basename(__FILE__))) {
            return;
        } else {
            if (!( isset($_GET['post']) || isset($_POST['post']) || ( isset($_REQUEST['action']) && 'lsd_duplicate_config' === sanitize_text_field(wp_unslash($_REQUEST['action'])) ) )) {// Input is ok.
                wp_die('No post to duplicate has been supplied!');
            }
        }

        /*
         * get the original post id
         */
        $post_id = ( isset($_GET['post']) ? absint($_GET['post']) : absint($_POST['post']) );

        /*
         * and all the original post data then
         */
        $post = get_post($post_id);

        /*
         * if you don't want current user to be the new post author,
         * then change next couple of lines to this: $new_post_author = $post->post_author;
         */
        $current_user = wp_get_current_user();
        $new_post_author = $current_user->ID;

        /*
         * if post data exists, create the post duplicate
         */
        if (isset($post) && null !== $post) {
            /*
             * new post data array
             */
            $args = array(
                'comment_status' => $post->comment_status,
                'ping_status' => $post->ping_status,
                'post_author' => $new_post_author,
                'post_content' => $post->post_content,
                'post_excerpt' => $post->post_excerpt,
                'post_name' => $post->post_name,
                'post_parent' => $post->post_parent,
                'post_password' => $post->post_password,
                'post_status' => 'draft',
                'post_title' => $post->post_title . esc_html__(' - copy', 'lsd'),
                'post_type' => $post->post_type,
                'to_ping' => $post->to_ping,
                'menu_order' => $post->menu_order,
            );

            /*
             * insert the post by wp_insert_post() function
             */
            $new_post_id = wp_insert_post($args);

            /*
             * get all current post terms ad set them to the new post draft
             */
            $taxonomies = get_object_taxonomies($post->post_type); // returns array of taxonomy names for post type, ex array("category", "post_tag");.
            foreach ($taxonomies as $taxonomy) {
                $post_terms = wp_get_object_terms($post_id, $taxonomy, array('fields' => 'slugs'));
                wp_set_object_terms($new_post_id, $post_terms, $taxonomy, false);
            }

            /*
             * duplicate all post meta just in two SQL queries
             */
            $post_meta_infos = $lsdb->get_results("SELECT meta_key, meta_value FROM $lsdb->postmeta WHERE post_id=$post_id");
            if (count($post_meta_infos) !== 0) {
                $sql_query = "INSERT INTO $lsdb->postmeta (post_id, meta_key, meta_value) ";
                foreach ($post_meta_infos as $meta_info) {
                    $meta_key = $meta_info->meta_key;
                    if ('_wp_old_slug' !== $meta_key) {
                        continue;
                    }
                    $meta_value = addslashes($meta_info->meta_value);
                    $sql_query_sel[] = "SELECT $new_post_id, '$meta_key', '$meta_value'";
                }
                $sql_query .= implode(' UNION ALL ', $sql_query_sel);
                $lsdb->query($sql_query);
            }

            /*
             * finally, redirect to the edit post screen for the new draft
             */
            wp_redirect(admin_url('post.php?action=edit&post=' . $new_post_id));
            exit();
        } else {
            wp_die('Post creation failed, could not find original post: ' . esc_attr($post_id));
        }
    }

    /**
     * Save the variation metas
     *
     * @param type $variation_id Variation id.
     */
    public function lsd_save_variation_settings_fields($variation_id) {
        $meta_key = 'lsd-metas';
        if (isset($_POST[$meta_key]) && wp_verify_nonce($_POST['lsd_save_variation_config_nonce'], basename(__FILE__))) {
            $variation = wc_get_product($variation_id);
            $old_metas = get_post_meta($variation->parent->id, $meta_key, true);
            if (empty($old_metas)) {
                $old_metas = array();
            }
            $new_metas = array_replace($old_metas, sanitize_text_field(wp_unslash($_POST[$meta_key]))); // Input var okay.
            update_post_meta($variation->parent->id, $meta_key, $new_metas);
        }
    }

    /**
     * Get the config selector for each product
     */
    public function lsd_get_product_config_selector() {
        $id = get_the_ID();

        $args = array(
            'post_type' => 'lsd-config',
            'nopaging' => true,
        );
        $configs = get_posts($args);
        $configs_ids = array('' => 'None');
        foreach ($configs as $config) {
            $configs_ids[$config->ID] = $config->post_title;
        }
        ?>
        <div id="lsd_config_data" class="show_if_simple">
            <?php
            $this->lsd_get_product_tab_row($id, $configs_ids, 'Configuration');
            ?>
        </div>
        <?php
    }

    /**
     * Add a new tab for the product to choose a config
     *
     * @param type $pid Product id.
     * @param type $configs_ids Configuration ids.
     * @param type $title Title.
     */
    private function lsd_get_product_tab_row($pid, $configs_ids, $title) {
        $begin = array(
            'type' => 'sectionbegin',
            'id' => 'lsd-metas-data',
        );

        $configurations = array(
            'title' => $title,
            'name' => "lsd-metas[$pid][config-id]",
            'type' => 'select',
            'options' => $configs_ids,
        );

        $end = array('type' => 'sectionend');
        $settings = apply_filters(
                'lsd_product_tab_settings', array(
            $begin,
            $configurations,
            $end,
                )
        );
        //Escaping output.
        $allowed_tags = lsd_allowed_tags();
        $raw_output = o_admin_fields($settings);
        wp_nonce_field('edit', 'lsd_save_product_config_nonce');
        echo "<div class='lsd-product-config-row'>" . wp_kses($raw_output, $allowed_tags) . '</div>';
    }

    /**
     * Set Variables product configuration form*
     *
     * @param type $loop Loop.
     * @param type $variation_data Variation data.
     * @param type $variation Variation object.
     */
    public function lsd_get_variation_product_config_selector($loop, $variation_data, $variation) {
        $id = $variation->ID;

        $args = array(
            'post_type' => 'lsd-config',
        );
        $configs = get_posts($args);
        $configs_ids = array('' => 'None');
        foreach ($configs as $config) {
            $configs_ids[$config->ID] = $config->post_title;
        }
        ?>
        <tr>
            <td>
                <?php
                $this->lsd_get_product_tab_row($id, $configs_ids, 'Configuration');
                ?>
            </td>
        </tr>
        <?php
    }

    /**
     *  Force the layout in the config to be one column
     *
     * @param array $columns Array of columns number.
     * @return int
     */
    public function lsd_get_layout_columns($columns) {
        $columns['lsd-config'] = 1;
        return $columns;
    }

}
