<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://designersuiteforwp.com
 * @since             1.1.0
 * @package           Lsd
 *
 * @wordpress-plugin
 * Plugin Name:       Lettering & Signs Designer for WooCommerce
 * Plugin URI:        https://designersuiteforwp.com
 * Description:       A simple lettering & signs design plugin for WooCommerce
 * Version:           2.0.0
 * Author:            ORION
 * Author URI:        https://designersuiteforwp.com
 * License:           GPL-2.0+
 * License URI:       https://designersuiteforwp.com
 * Text Domain:       lsd
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

define( 'LSD_VERSION', '2.0.0' );
define( 'LSD_URL', plugins_url('/', __FILE__) );
define( 'LSD_DIR', dirname(__FILE__) );
define( 'LSD_MAIN_FILE', 'lettering-signs-designer/lsd.php' );
define( 'LSD_PLUGIN_NAME', 'Lettering & Signs Designer for WooCommerce' );
define( 'LSD_PLUGIN_ID', 001 );


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-lsd-activator.php
 */
function activate_lsd() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-lsd-activator.php';
	Lsd_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-lsd-deactivator.php
 */
function deactivate_lsd() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-lsd-deactivator.php';
	Lsd_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_lsd' );
register_deactivation_hook( __FILE__, 'deactivate_lsd' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-lsd.php';
require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'class-lsd-config.php' );

if(!function_exists("o_admin_fields"))
{
    require_once plugin_dir_path(__FILE__) . 'includes/utils.php';
}

require_once plugin_dir_path(__FILE__) . 'includes/functions.php';

//include default skin
require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'skins' . DIRECTORY_SEPARATOR . 'default' . DIRECTORY_SEPARATOR . 'class-lsd-default-skin.php' );

require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'class-lsd-editor.php' );

if(!class_exists("TCPDF")){
    require_once( LSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'tcpdf' . DIRECTORY_SEPARATOR . 'tcpdf.php' );
}


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_lsd() {

	$plugin = new Lsd();
	$plugin->run();

}
run_lsd();
