<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://designersuiteforwp.com
 * @since      1.0.0
 *
 * @package    Lsd
 * @subpackage Lsd/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Lsd
 * @subpackage Lsd/public
 * @author     ORION <support@designersuiteforwp.com>
 */
class Lsd_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        wp_enqueue_style('spectrum-style', LSD_URL . 'public/libs/spectrum/spectrum.css', array(), $this->version, 'all');
        wp_enqueue_style('lsd-tooltipster-style', LSD_URL . 'public/libs/tooltipster/tooltipster.bundle.min.css', array(), $this->version, 'all');
        wp_enqueue_style('bxslider-style', LSD_URL . 'public/libs/bxslider/jquery.bxslider.min.css', array(), $this->version, 'all');
        wp_enqueue_style('themify-icons', LSD_URL . 'public/libs/themify/themify-icons.css', array(), $this->version, 'all');
        wp_enqueue_style('fontawesome-icons', LSD_URL . 'public/libs/fontawesome/css/all.css', array(), $this->version, 'all');
        wp_enqueue_style($this->plugin_name, LSD_URL . 'public/css/lsd-public.css', array(), $this->version, 'all');
        wp_enqueue_style('lsd-tooltip-public-css', LSD_URL . 'public/css/tooltip.min.css', array(), $this->version, 'all');
        wp_enqueue_style('lsd-bs-modal-css', LSD_URL . 'public/js/modal/modal.min.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        wp_enqueue_script('spectrum-script', LSD_URL . 'public/libs/spectrum/spectrum.js', array('jquery'), $this->version, false);
        wp_enqueue_script('bxslider-script', LSD_URL . 'public/libs/bxslider/jquery.bxslider.min.js', array('jquery'), $this->version, false);
        wp_enqueue_script('lsd-tooltip-public-js', LSD_URL . 'public/js/tooltip.min.js', array('jquery'), $this->version, false);
        wp_enqueue_script('lsd-tooltipster-script', LSD_URL . 'public/libs/tooltipster/tooltipster.bundle.min.js', array('jquery'), $this->version, false);
        wp_dequeue_script('bs-modal');
        wp_enqueue_script('bs-modal', LSD_URL . 'public/js/modal/lsd-modal.min.js', array('jquery'), $this->version, false);
        wp_enqueue_script($this->plugin_name, LSD_URL . 'public/js/lsd-public.js', array('jquery'), $this->version, false);
        wp_localize_script($this->plugin_name, 'lsd_ajax_object', array('lsd_ajax_url' => admin_url('admin-ajax.php')));
    }

    /**
     * Register the plugin shortcodes
     */
    public function lsd_register_shortcodes() {
        add_shortcode('lsd-editor', array($this, 'lsd_get_editor_shortcode_handler'));
        add_shortcode('lsd_editor', array($this, 'lsd_get_editor_shortcode_handler'));
        add_shortcode('lsd_builder', array($this, 'lsd_get_editor_shortcode_handler'));
        add_shortcode('lsd', array($this, 'lsd_get_editor_shortcode_handler'));
    }

    /**
     * Callback for the shortcode lsd-editor
     */
    public function lsd_get_editor_shortcode_handler($atts) {
        global $wp_query;
        $product_id = get_query_var("product_id", false);

//Maybe the product ID is included in the shortcode
        if (isset($atts["product"])) {
            $product = $atts["product"];
        }
        if (!$product_id) {
            $product_id = $product;
        }

        if (!$product_id) {
            return esc_html__("You're trying to access the customization page whitout a product to customize. This page should only be accessed using one of the customization buttons.", 'lsd');
        }

        $editor_obj = new LSD_Editor($product_id);
        $raw_output = $editor_obj->get_editor();
        $output = apply_filters("lsd_output_editor", $raw_output, $product_id);
        return $output;
    }

    /**
     * Function to set variables filters.
     * @global type $lsd_settings
     */
    public function lsd_set_variable_action_filters() {
        global $lsd_settings;
        $options = $lsd_settings['lsd-general-options'];
        $woo_version = $this->lsd_get_woo_version_number();
        if ('name' === $options['lsd-parts-position-cart']) {
            if ($woo_version < 2.1) {
//Old WC versions
                add_filter('woocommerce_in_cart_product_title', array($this, 'lsd_get_data'), 10, 3);
            } else {
//New WC versions
                add_filter('woocommerce_cart_item_name', array($this, 'lsd_get_data'), 10, 3);
            }
        } else {
            if ($woo_version < 2.1) {
//Old WC versions
                add_filter('woocommerce_in_cart_product_thumbnail', array($this, 'lsd_get_data'), 10, 3);
            } else {
//New WC versions
                add_filter('woocommerce_cart_item_thumbnail', array($this, 'lsd_get_data'), 10, 3);
            }
        }
        $append_content_filter = $options['lsd-content-filter'];

        if ('0' !== $append_content_filter && !is_admin()) {

            add_filter('the_content', array($this, 'lsd_filter_content'), 99);
        }
    }

    /**
     * Display the customizer in the proper page
     */
    public function lsd_filter_content($content) {
        global $lsd_settings;
        global $wp_query;
        $options = $lsd_settings['lsd-general-options'];
        $lsd_page_id = $options['lsd_page_id'];
        if (function_exists('icl_object_id')) {
            $lsd_page_id = icl_object_id($lsd_page_id, 'page', false, ICL_LANGUAGE_CODE);
        }
        $current_page_id = get_the_ID();
        if (intval($lsd_page_id) === intval($current_page_id)) {
            $item_id = $wp_query->query_vars['product_id'];
            $editor_obj = new LSD_Editor($item_id);
            $raw_output = $editor_obj->get_editor();
            $output = apply_filters("lsd_output_editor", $raw_output, $item_id);
            $content .= $output;
        }

        return $content;
    }

    /**
     * Retrieve lsd data in cart
     *
     * @param type $thumbnail_code Thumbnail code.
     * @param type $values Values to be added in the cart.
     * @param type $cart_item_key Cart item key.
     * @return string
     */
    public function lsd_get_cart_data($thumbnail_code, $values, $cart_item_key) {
        $variation_id = $values['product_id'];
        if (isset($values['lsd_generated_data']) && isset($values['lsd_generated_data']['output'])) {
            $thumbnail_code .= '<br>';
            $image = $values['lsd_generated_data']['output']['files'];
            $thumbnail_code .= '<img src="' . $image . '">';
        }
        return $thumbnail_code;
    }

    /**
     * Add lsd query vars
     *
     * @param type $a_vars Query variables.
     * @return string
     */
    public function lsd_add_query_vars($a_vars) {
        $a_vars[] = 'product_id';
        $a_vars[] = 'tpl';
        $a_vars[] = 'edit';
        $a_vars[] = 'design_index';
        $a_vars[] = 'oid';
        return $a_vars;
    }

    /**
     * Add lsd query vars
     *
     * @global type $lsd_settings
     * @global type $wp_rewrite
     * @param type $param Params.
     */
    public function lsd_add_rewrite_rules($param) {
        global $lsd_settings;
        global $wp_rewrite;
        $options = $lsd_settings['lsd-general-options'];
        $lsd_page_id = $options['lsd_page_id'];
        if (function_exists('icl_object_id')) {
            $lsd_page_id = icl_object_id($lsd_page_id, 'page', false, ICL_LANGUAGE_CODE);
        }
        $lsd_page = get_post($lsd_page_id);
        if (is_object($lsd_page)) {
//$slug = $lsd_page->post_name;
            $raw_slug = get_permalink($lsd_page->ID);
            $home_url = home_url('/');
            $slug = str_replace($home_url, '', $raw_slug);
//If the slug does not have the trailing slash, we get 404 (ex postname = /%postname%)
            $sep = '';
            if (substr($slug, -1) !== '/') {
                $sep = '/';
            }
//            var_dump(substr($slug, -1));
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'design/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&tpl=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'edit/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&edit=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'ordered-design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&oid=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );

            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'saved-design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&design_index=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );

            $wp_rewrite->flush_rules(false);
        }
    }

    /**
     * Get design url
     *
     * @param type $post_id Product id.
     * @param type $product Product object.
     */
    public function get_design_from_blank_urls($post_id, $product) {
        $design_from_blank_urls = array();
        if ('variable' === $product->get_type()) {
            $variations = $product->get_available_variations();
            foreach ($variations as $variation) {
                $variation_id = $variation['variation_id'];
                $lsd_product = new LSD_Product($variation_id);
                $design_from_blank_urls[$variation_id] = $lsd_product->lsd_get_design_url();
            }
        } else {
            $lsd_product = new LSD_Product($post_id);
            $design_from_blank_urls[$post_id] = $lsd_product->lsd_get_design_url();
        }
        ?>
        <script>
            var design_from_blank_urls =<?php echo wp_json_encode($design_from_blank_urls); ?>;
        </script>
        <?php
    }

    /**
     * Get custom button on product page
     */
    public function lsd_get_customize_btn() {
        $product_id = get_the_ID();
        $lsd_product = new LSD_Product($product_id);
        $allowed_tags = lsd_allowed_tags();
        $raw_output = $lsd_product->lsd_get_buttons();
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Get custom button for each element of shop page
     *
     * @global type $lsd_settings Array of settings.
     * @param type $html Html output.
     * @param type $product Product object.
     * @return type
     */
    public function lsd_get_customize_btn_loop($html, $product) {
        global $lsd_settings;
        $general_options = $lsd_settings['lsd-general-options'];
        $hide_buttons_shop_page = get_proper_value($general_options, 'lsd-hide-btn-shop-pages', 0);
        $hide_cart_button_shop_page = get_proper_value($general_options, 'lsd-hide-cart-shop-pages', 0);
        $product_id = $product->get_id();
        $lsd_product = new LSD_Product($product_id);
        $product_class = get_class($product);
        if ('WC_Product_Simple' === $product_class && $lsd_product->is_customizable()) {
            if (!$hide_buttons_shop_page) {
                $html .= $lsd_product->lsd_get_buttons();
            }

            if ($hide_cart_button_shop_page) {
                $html .= '<script>
                    jQuery(".add_to_cart_button[data-product_id=' . $product_id . ']").hide();
                </script>';
            } else {
                
            }
        }
        return $html;
    }

    /**
     * Add classes to body
     */
    public function lsd_add_class_to_body($classes) {
        global $wp_query;
        if (isset($wp_query->query_vars['product_id'])) {
            array_push($classes, 'lsd-customization-page lsd-product-' . $wp_query->query_vars['product_id']);
        }
        return $classes;
    }

    /**
     * Add class to item
     */
    public function lsd_get_item_class($classes, $class, $post_id) {
        global $lsd_settings;
        $general_options = $lsd_settings['lsd-general-options'];
        $hide_cart_button = get_proper_value($general_options, 'lsd-hide-cart-button', true);
        $hide_cart_button_shop_page = get_proper_value($general_options, 'lsd-hide-cart-shop-pages', 0);

        if (in_array('product', $classes, true)) {
            $lsd_product = new LSD_Product($post_id);
            if ($lsd_product->is_customizable()) {
                array_push($classes, 'lsd-is-customizable');
            }

            if ($hide_cart_button) {
                array_push($classes, 'lsd-hide-cart-button');
            }

            if ($hide_cart_button_shop_page) {
                array_push($classes, 'lsd-hide-cart-button-on-shop');
            }
        }
        return $classes;
    }

    /*
     * Ajax callback that controls if design is already added or not
     */

    public function add_custom_design_to_cart_ajax() {
        global $woocommerce;
        $message = '';
        if (lsd_woocommerce_version_check()) {
            $cart_url = wc_get_cart_url();
        } else {
            $cart_url = $woocommerce->cart->get_cart_url();
        }

        if (isset($_POST['data']) && wp_verify_nonce($_POST['lsd_generate_design_nonce'], 'lsd_generate_design')) {
            $final_data = $_POST['data'];
            $cart_item_key = false;

            $tmp_dir = uniqid();
            $upload_dir = wp_upload_dir();
            $generation_path = $upload_dir['basedir'] . "/LSD/$tmp_dir";
            $generation_url = $upload_dir['baseurl'] . "/LSD/$tmp_dir";
            $zip_name = uniqid("lsd_") . ".zip";

            $result = $this->export_data_to_files($generation_path, $generation_url, $final_data, $zip_name);
            $final_data['output']['files'] = $result;
            $final_data['output']['working_dir'] = $tmp_dir;
            $final_data['output']['tpl'] = $final_data['tpl'];
            $final_data['product_id'] = $final_data['product_id'];
            $final_data["output"]["zip"] = $zip_name;
            $final_data['quantity'] = $final_data['quantity'];
            $newly_added_cart_item_key = true;
            if ($cart_item_key) {
                $woocommerce->cart->cart_contents[$cart_item_key]['lsd_generated_data'] = $final_data;
                $woocommerce->cart->calculate_totals();
                $message = "<div class='lsd_notification success f-right'>" . esc_html__('Item successfully updated.', 'lsd') . " <a href='$cart_url'>" . esc_html__('View Cart', 'lsd') . '</a></div>';
            } else {
                $newly_added_cart_item_key = $this->add_design_to_cart($final_data);
                if ($newly_added_cart_item_key) {
                    do_action('lsd_add_custom_data_to_cart', $newly_added_cart_item_key);
                    $message = "<div class='lsd_notification success f-right'>" . esc_html__('Product successfully added to basket.', 'lsd') . " <a href='$cart_url'>View Cart</a></div>";
                } else {
                    $message = "<div class='lsd_notification failure f-right'>" . esc_html__('A problem occured while adding the product to the cart. Please try again.', 'lsd') . '</div>';
                }
            }
            echo wp_json_encode(
                    array(
                        'success' => $newly_added_cart_item_key,
                        'message' => $message,
                        'url' => $cart_url,
                    )
            );
        }
        die();
    }

//actual function to add the design to cart
    private function add_design_to_cart($final_data) {
        global $woocommerce;

        $variation_id = $final_data['product_id'];
        $quantity = $final_data['quantity'];
        $product = wc_get_product($variation_id);
        $variation = array();
        if ('simple' === $product->get_type()) {
            $product_id = $variation_id;
        } else {
            $variation = $product->get_variation_attributes();
            $product_id = $product->get_parent_id();
        }

        $newly_added_cart_item_key = $woocommerce->cart->add_to_cart($product_id, $quantity, $variation_id, $variation, array('lsd_generated_data' => $final_data));
        if (method_exists($woocommerce->cart, 'maybe_set_cart_cookies')) {
            $woocommerce->cart->maybe_set_cart_cookies();
        }
        return $newly_added_cart_item_key;
    }

    /*
     * Retrieve woocommerce version
     *
     */

    private function lsd_get_woo_version_number() {
// If get_plugins() isn't available, require it
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

// Create the plugins folder and file variables
        $plugin_folder = get_plugins('/woocommerce');
        $plugin_file = 'woocommerce.php';

// If the plugin version number is set, return it
        if (isset($plugin_folder[$plugin_file]['Version'])) {
            return $plugin_folder[$plugin_file]['Version'];
        } else {
// Otherwise return null
            return null;
        }
    }

//ajax callback to retrieve data needed to generated image
    public function lsd_generate_image_ajax() {
        if (isset($_POST['data']) && wp_verify_nonce($_POST['lsd_generate_design_nonce'], 'lsd_generate_design')) {
            $data = $_POST['data'];
            if (is_user_logged_in()) {
                $current_user = wp_get_current_user();
                $userid = $current_user->ID;
                $transient = 'lsd-id' . $userid;
                if (!get_transient($transient)) {
                    $name = uniqid('lsd-');
                    set_transient($transient, $name, 12 * HOUR_IN_SECONDS);
                } else {
                    $name = get_transient($transient);
                }
            } else {
                if (!session_id()) {
                    session_start();
                }
                if (!isset($_SESSION['lsd-user-id'])) {
                    $name = uniqid('lsd-');
                    $_SESSION['lsd-user-id'] = $name;
                } else {
                    $name = $_SESSION['lsd-user-id'];
                }
            }

            $upload_dir = wp_upload_dir();
            if (!file_exists($upload_dir['basedir'] . '/LSD')) {
                mkdir($upload_dir['basedir'] . '/LSD', 0777, true);
            }

            if (!isset($data['font']) || '' === $data['font']) {
                $response = array(
                    'message' => 'No font selected. Make sure you have defined some fonts to be used. ',
                );
                echo wp_json_encode($response);
                die();
            }

            if (!isset($data['canvas_color'])) {
                $data['canvas_color'] = '';
            }

            if (!isset($data['shadow_color'])) {
                $data['shadow_color'] = "rgb(132,132,132)";
            }

            if (!isset($data['outline_color'])) {
                $data['outline_color'] = "rgb(132,132,132)";
            }


            if (!isset($data['font-url'])) {
                $data['font-url'] = "";
            }

            if (!isset($data['font-ttf'])) {
                $data['font-ttf'] = "";
            }

            if (!isset($data['alignment'])) {
                $data['alignment'] = 'left';
            }

            if (!isset($data['spacing'])) {
                $data['spacing'] = 1;
            }

            /*
              if (!isset($data['curving'])) {
              $data['curving'] = 0;
              } */

            if (!isset($data['shadow'])) {
                $data['shadow'] = 0;
            }

            if (!isset($data['outline'])) {
                $data['outline'] = 0;
            }

            if (!isset($data['dimensions-type'])) {
                $data['dimensions-type'] = 'canvas-dimensions';
            }

            if (!isset($data['font-size'])) {
                $data['font-size'] = 0;
            }


//replace line break with proper php symbol
            $text = $data['text'];
            if ('' === $text) {
                $text = 'NO TEXT';
            }

            $data['text'] = $text;

            if (file_exists($upload_dir['basedir'] . '/LSD/' . $name . '.png')) {
                unlink($upload_dir['basedir'] . '/LSD/' . $name . '.png');
            }

            $output_path = $upload_dir['basedir'] . '/LSD/' . $name . '.png';
            $data['path'] = $output_path;
            $output_url = $upload_dir['baseurl'] . '/LSD/' . $name . '.png';
            $data['url'] = $output_url;

            $response = $this->lsd_generate_image($data);
            echo wp_json_encode($response);
            die();
        }
        die();
    }

    /**
     * Actual function that generate the image with all data needed
     *
     * @param type $data Array of parameters
     * @return string
     */
    public function lsd_generate_image($data) {
        if (!function_exists('gd_info')) {
            $response = array(
                'message' => 'Install php5-gd first: http://php.net/manual/fr/image.setup.php',
            );
            return $response;
        }
        $gd = gd_info();
        if ($gd["FreeType Support"] == false) {
            $response = array(
                'message' => 'Compile php5-gd with FreeType 2 support.',
            );
            return $response;
        }

        $message = "";
        $unit = $data['unit'];
        $text = $data['text'];

        $text_color = $data['text_color'];
        $bg_color = $data['canvas_color'];


        $alignement = $data['alignment'];
        
        $spacing = $data['spacing'];

//shadow
        $shadow = intval($data['shadow']);
        $shadow_color = $data['shadow_color'];


//outline
        $outline = intval($data['outline']);
        $outline_color = $data['outline_color'];

        $font_ttf = $data['font-ttf'];

        if ($data['style'] === 'bold') {
            $fonturl = $data['font-url'] . ":700";
        } elseif ($data['style'] === 'italic') {
            $fonturl = $data['font-url'] . ":400i";
        } elseif ($data['style'] === 'both') {
            $fonturl = $data['font-url'] . ":700i";
        } else {
            $fonturl = $data['font-url'];
        }

        if ('' !== $fonturl && '' === $font_ttf) {
            $font_ttf = lsd_retrieve_ttf_from_url($fonturl);
            if ($font_ttf === '') {
                $response = array(
                    'message' => __("Could'nt retrieve the ttf file associated with the selected font url. Please retry.", "lsd"),
                );
                return $response;
            }
        }
        $output_path = $data['path'];
        $output_url = $data['url'];

        $original_width = $data['custom-width'];
        $original_height = $data['custom-height'];


        $dimensions_type = $data['dimensions-type'];
//if dimensions type is font-size, calculate the appropiate value of the text font size
        $font_size = lsd_convert_unit_to_px($data['font-size'], $unit);

        $ratio = 1;

        if ('font-size' === $dimensions_type) {
            if ($font_size > 1000) {
                $ratio = $font_size / 1000;
                $font_size = 1000;
            }
            $bbox = imageftbbox($font_size, 0, $font_ttf, $text);

            // Transform coordinates into width+height and position
            $ascender = abs($bbox[7]);
            $descender = abs($bbox[1]);
            $size_w = abs($bbox[2] - $bbox[0]);
            if ($bbox[0] < -1) {
                $size_w = abs($bbox[2]) + abs($bbox[0]) - 1;
            }
            $size_h = $ascender + $descender;
            $x = -$bbox[0];
            $y = $ascender;
            if ($spacing > 0) {
                for ($i = 0; $i < strlen($text); $i++) {
                    $size_w += $spacing + abs($bbox[6]);
                }
            }
        } else {
            
            if (!is_null($original_width)) {
                $size_w = lsd_convert_unit_to_px($original_width, $unit);
            } else {
                $size_w = 130;
            }

            if (!is_null($original_height)) {
                $size_h = lsd_convert_unit_to_px($original_height, $unit);
            } else {
                $size_h = 44;
            }

            $ratio = $size_w / 3000;
            $size_w = 3000;
            $size_h = $size_h / $ratio;

            $overall_width = $overall_height = $font_size = 0;

//if text is aligned to center, find proper font size.
            if ("center" === $alignement) {
                $text_height = $txt_width = 0;
                $overall_width = intval($size_w);
                $overall_height = intval($size_h);
                while ($txt_width <= $overall_width && $text_height <= $overall_height) {
                    $font_size++;
                    $bbox = imagettfbbox($font_size, 0, $font_ttf, $text);

                    $txt_width = abs($bbox[0]) + abs($bbox[2]);
                    $ascender = abs($bbox[7]);
                    $descender = abs($bbox[1]);
                    $text_height = $ascender + $descender;
                    if ($spacing > 0) {
                        for ($i = 0; $i < strlen($text); $i++) {
                            $txt_width += $spacing + abs($bbox[6]);
                        }
                    }
                }

                /*
                  if($font_size > 1000){

                  $ratio = $font_size / 1000;
                  $font_size = 1000;
                  $bbox = imagettfbbox($font_size, 0, $font_ttf, $text);
                  $txt_width = abs($bbox[0]) + abs($bbox[2]);
                  $ascender = abs($bbox[7]);
                  $descender = abs($bbox[1]);
                  $text_height = $ascender + $descender;
                  if ($spacing > 0) {
                  for ($i = 0; $i < strlen($text); $i++) {
                  $txt_width += $spacing + abs($bbox[6]);
                  }
                  }

                  $size_w = $size_w / $ratio;
                  $size_h = $size_h / $ratio;
                  } */

// now we get the center position
                $x = ($size_w - $txt_width) / 2;
                $y = $ascender;
            } elseif ("left" == $alignement) {
                while ($overall_height < $size_h && $overall_width < $size_w) {
                    $font_size ++;
// Get bounding box
                    $bbox = imageftbbox($font_size, 0, $font_ttf, $text);
// Transform coordinates into width+height and position
                    $ascender = abs($bbox[7]);
                    $descender = abs($bbox[1]);
                    $overall_width = abs($bbox[0]) + abs($bbox[2]);
                    $overall_height = $ascender + $descender;
                    $x = -$bbox[0];
                    $y = $ascender;

                    if ($spacing > 0) {
                        for ($i = 0; $i < strlen($text); $i++) {
                            $overall_width += $spacing + abs($bbox[6]);
                        }
                    }
                }

                /*
                  if($font_size > 1000){
                  $ratio = $font_size / 1000;
                  $font_size = 1000;
                  $bbox = imageftbbox($font_size, 0, $font_ttf, $text);
                  // Transform coordinates into width+height and position
                  $ascender = abs($bbox[7]);
                  $descender = abs($bbox[1]);
                  $overall_width = abs($bbox[0]) + abs($bbox[2]);
                  $overall_height = $ascender + $descender;
                  $x = -$bbox[0];
                  $y = $ascender;

                  if ($spacing > 0) {
                  for ($i = 0; $i < strlen($text); $i++) {
                  $overall_width += $spacing + abs($bbox[6]);
                  }
                  }
                  $size_w = $size_w / $ratio;
                  $size_h = $size_h / $ratio;
                  } */
            } else {
                
                while ($overall_height < $size_h && $overall_width < $size_w) {
                    $font_size ++;
// Get bounding box
                    $bbox = imageftbbox($font_size, 0, $font_ttf, $text);
// Transform coordinates into width+height and position
                    $ascender = abs($bbox[7]);
                    $descender = abs($bbox[1]);
                    $overall_width = abs($bbox[0]) + abs($bbox[2]);
                    $overall_height = $ascender + $descender;
                    $x = $size_w - $overall_width;
                    $y = $ascender;

                    if ($spacing > 0) {
                        for ($i = 0; $i < strlen($text); $i++) {
                            $overall_width += $spacing + abs($bbox[6]);
                        }
                    }
                }

                /*
                  if($font_size > 1000){
                  $ratio = $font_size / 1000;
                  $font_size = 1000;

                  $bbox = imageftbbox($font_size, 0, $font_ttf, $text);
                  // Transform coordinates into width+height and position
                  $ascender = abs($bbox[7]);
                  $descender = abs($bbox[1]);
                  $overall_width = abs($bbox[0]) + abs($bbox[2]);
                  $overall_height = $ascender + $descender;
                  $x = $size_w - $overall_width;
                  $y = $ascender;

                  if ($spacing > 0) {
                  for ($i = 0; $i < strlen($text); $i++) {
                  $overall_width += $spacing + abs($bbox[6]);
                  }
                  }
                  $size_w = $size_w / $ratio;
                  $size_h = $size_h / $ratio;
                  } */
            }
        }

// Create image
        $im = imagecreatetruecolor($size_w, $size_h);

// Allocate background color
        if ('' === $bg_color) {
            imagealphablending($im, false);
            imagesavealpha($im, true);
            //setting completely transparent color
            $bg_color = imagecolorallocate($im, 0, 0, 0);;
            imagecolortransparent($im, $bg_color);
        } else {
            try {
                $bg_int = hexdec($bg_color);
            } catch (Exception $e) {
                $response = array(
                    'message' => '<br/>' . $e->getMessage() . '<br/>',
                );
                return $response;
            }

            $bg_arr = array("red" => 0xFF & ($bg_int >> 0x10),
                "green" => 0xFF & ($bg_int >> 0x8),
                "blue" => 0xFF & $bg_int);

            $bgcol = ImageColorAllocate($im, $bg_arr["red"], $bg_arr["green"], $bg_arr["blue"]);

            //Fill image with background colour
            try {
                imagefilledrectangle($im, 0, 0, $size_w - 1, $size_h - 1, $bgcol);
            } catch (Exception $e) {
                $response = array(
                    'message' => '<br/>' . $e->getMessage() . '<br/>',
                );
                return $response;
            }
        }

        /*
         * Add text shadow
         */
        if ($shadow > 0) {
            lsd_imagettftextshadow($im, $font_size, $x, $y, $shadow_color, $font_ttf, $text, $shadow, $spacing);
        }

        /*
         * Add text outline
         */
        if ($outline > 0) {
            lsd_imagettftextoutline($im, $font_size, $x, $y, $outline_color, $font_ttf, $text, $outline, $spacing);
        }

// Allocate text color
        try {
                $text_int = hexdec($text_color);
            } catch (Exception $e) {
                $response = array(
                    'message' => '<br/>' . $e->getMessage() . '<br/>',
                );
                return $response;
            }
        $col_arr = array("red" => 0xFF & ($text_int >> 0x10),
                "green" => 0xFF & ($text_int >> 0x8),
                "blue" => 0xFF & $text_int);
            
        $textcol = ImageColorAllocate($im, $col_arr["red"], $col_arr["green"], $col_arr["blue"]);

// Render text into image
        if ($spacing <= 0) {
            imagefttext($im, $font_size, 0, $x, $y, $textcol, $font_ttf, $text);
        } else {
            $temp_x = $x;
            for ($i = 0; $i < strlen($text); $i++) {
                $bbox = imagefttext($im, $font_size, 0, $temp_x, $y, $textcol, $font_ttf, $text[$i]);
                $temp_x = abs($bbox[2]) + $spacing;
            }
        }

// Convert true colour image (needed for above) to palette image
        imagetruecolortopalette($im, FALSE, 256);

// Output created image

        if (!isset($data['format'])) {
            $format = "png";
        } else {
            $format = $data['format'];
        }

        if ('true' === $data['reverse']) {
            imageflip($im, IMG_FLIP_HORIZONTAL);
        }

        if ($format === "png") {
            header('Content-Type: image/png');
            imagepng($im, $output_path, 9);
        }

        if ($format === "jpg") {
            header('Content-Type: image/jpeg');
            imagejpeg($im, $output_path, 100);
        }

        imagedestroy($im);

        $final_width = lsd_convert_px_to_unit(($size_w * $ratio), $unit);
        $scaled_width = lsd_convert_px_to_unit($size_w, $unit);

        $final_height = lsd_convert_px_to_unit(($size_h * $ratio), $unit);
        $scaled_height = lsd_convert_px_to_unit($size_h, $unit);

        $converted_descender = lsd_convert_px_to_unit(( $descender * $ratio), $unit);
        $scaled_descender = lsd_convert_px_to_unit($descender, $unit);

        $baseline = lsd_convert_px_to_unit(($ascender * $ratio), $unit);
        $scaled_baseline = lsd_convert_px_to_unit($ascender, $unit);

        $converted_font_size = lsd_convert_px_to_unit(($font_size * $ratio), $unit);
        $response = array(
            'width' => round($final_width),
            'scaledWidth' => round($scaled_width),
            'height' => round($final_height),
            'scaledHeight' => round($scaled_height),
            'descender' => $converted_descender,
            'scaledDescender' => $scaled_descender,
            'baseline' => $baseline,
            'scaledBaseline' => $scaled_baseline,
            'fontsize' => $converted_font_size,
            'image' => $output_url,
            'message' => $message,
        );

        return $response;
    }

    /**
     * Display user selection in cart
     *
     * @param type $thumbnail_code Thumbnail code.
     * @param type $cart_item Cart item.
     * @param type $cart_item_key Cart item key.
     * @return string
     */
    public function lsd_get_data($thumbnail_code, $cart_item, $cart_item_key) {
        if (isset($cart_item['lsd_generated_data'])) {
            $data = $cart_item['lsd_generated_data'];
            $formatted_config = $this->lsd_get_formatted_data($data);
            $thumbnail_code .= "<div class='lsd-cart-config o-wrap'><div class='o-col xl-1-1'>" . $formatted_config . '</div> </div>';
            $thumbnail_code .= '<br>';
            $original_part_img_url = '';
            if (isset($data['output']['files']['thumb']['image'])) {
                $original_part_img_url = $data['output']['files']['thumb']['image'];
            } else {
                if (isset($data['output']['files']['thumb']['image'])) {
                    $original_part_img_url = $data['output']['files']['full']['image'];
                }
            }

            $modal_id = $cart_item_key;

            $thumbnail_code .= '<span><a class="button lsd-view-design" data-toggle="o-modal" data-target="#' . $modal_id . '">View design</a></span>';
            $thumbnail_code .= '<div class="omodal fade o-modal lsd-modal lsd_part" id="' . $modal_id . '" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
                                    <div class="omodal-dialog">
                                      <div class="omodal-content">
                                        <div class="omodal-header">
                                          <button type="button" class="close" data-dismiss="omodal" aria-hidden="true">&times;</button>
                                          <h4 class="omodal-title">' . esc_html__('Preview', 'lsd') . '</h4>
                                        </div>
                                        <div class="omodal-body txt-center">
                                            <div><img src="' . $original_part_img_url . '"></div>
                                        </div>
                                      </div>
                                    </div>
                                  </div>';
        }
        echo $thumbnail_code;
    }

    /**
     * Returns the formatted recap.
     * @param type $recap
     * @return string
     */
    public function lsd_get_formatted_data($recap) {
        $output = "<div class='lsd-cart-options-container'>";
        $option = '';
        $filtered_recap = apply_filters('lsd_filter_recap', $recap);

        if (is_array($filtered_recap)) {
            foreach ($filtered_recap as $component => $option) {
                if ('font-url' !== $component && 'font-ttf' !== $component && 'output' !== $component && 'quantity' !== $component && 'output' !== $component && 'tpl' !== $component && 'product_id' !== $component && 'width' !== $component && 'height' !== $component && 'format' !== $component) {
                    $option = stripslashes($option);
                    $parts = array('custom-', '_');
                    $component = str_replace($parts, ' ', $component);
                    $output .= "<div><strong>$component: </strong>$option</div>";
                }
            }
        }
        $output .= '</div>';
        return $output;
    }

    /**
     *
     * @global type $lsd_settings
     * @param type $item_id Item id.
     * @param type $order_item Oder item object.
     * @param type $order_id Order id.
     */
    public function save_customized_item_meta($item_id, $order_item, $order_id) {
        global $lsd_settings;
        $output_options = get_proper_value($lsd_settings, 'lsd-output-options', array());
        $use_order_id = get_proper_value($output_options, 'use-order-id-as-zip-name', 'no');
        if ('yes' === $use_order_id) {
            $upload_dir = wp_upload_dir();
            $old_tmp_dir = $order_item->legacy_values['lsd_generated_data']['output']['working_dir'];
            $old_generation_path = $upload_dir["basedir"] . "/LSD/$old_tmp_dir";
            $new_generation_path = $upload_dir["basedir"] . "/LSD/$order_id-$item_id";


//Suppression ancien zip
            unlink($old_generation_path . "/" . $order_item->legacy_values["lsd_generated_data"]["output"]["zip"]);

//renommage pdf
            rename($old_generation_path, $new_generation_path);

//Regénération du zip
            $new_zip_name = "$order_id-$item_id.zip";
            $this->generate_design_archive($new_generation_path, "$new_generation_path/$new_zip_name");

//MAJ des metas
            $order_item->legacy_values['lsd_generated_data']['output']['working_dir'] = "$order_id-$item_id";
            $order_item->legacy_values["lsd_generated_data"]["output"]["zip"] = $new_zip_name;
        }
        if (isset($order_item->legacy_values['lsd_generated_data'])) {
            wc_add_order_item_meta($item_id, 'lsd_data', $order_item->legacy_values['lsd_generated_data']);
        }
    }

    /**
     *
     * @global type $lsd_settings
     * @param type $item_id Item id.
     * @param type $item Item object.
     * @param type $_product Product object.
     * @return type
     */
    public function lsd_get_order_custom_admin_data($item_id, $item, $product) {
        global $lsd_settings;
        $output_options = get_proper_value($lsd_settings, 'lsd-output-options', array());
        $design_composition_visible = get_proper_value($output_options, 'design-composition', 'no');
        $output = '';
        if (!is_object($item)) {
            return;
        }

        if (isset($item['lsd_data'])) {
            $upload_dir = wp_upload_dir();
            $data = $item['lsd_data'];
            if ('yes' === $design_composition_visible) {
                $formatted_config = $this->lsd_get_formatted_data($data);
                $output .= "<div class='lsd-order-config o-wrap'><div class='o-col xl-1-1'>" . $formatted_config . '</div> </div>';
                $output .= '<br>';
            }

            $tmp_dir = $data["output"]["working_dir"];
            $modal_id = $item_id;
            $output .= '<span><a class="button lsd-view-design" data-toggle="o-modal" data-target="#' . $modal_id . '">View design</a></span>';
            $output .= '<div class="omodal fade lsd-modal o-modal lsd_part" id="' . $modal_id . '" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
                                    <div class="omodal-dialog">
                                      <div class="omodal-content">
                                        <div class="omodal-header">
                                          <button type="button" class="close" data-dismiss="omodal" aria-hidden="true">&times;</button>
                                          <h4 class="omodal-title">' . esc_html__('Preview', 'lsd') . '</h4>
                                        </div>
                                        <div class="omodal-body txt-center">
                                            <div><img src="' . $upload_dir["baseurl"] . '/LSD/' . $tmp_dir . '/design.png"></div>
                                        </div>
                                      </div>
                                    </div>
                                  </div>';

            $zip_file = $data["output"]["zip"];

            if ($upload_dir["baseurl"] . "/LSD/$tmp_dir/$zip_file") {
                $output .= "<a class='button' href='" . $upload_dir["baseurl"] . "/LSD/$tmp_dir/$zip_file' download='" . basename($zip_file) . "'>" . esc_html__("Download design", "lsd") . "</a> ";
            } elseif ($upload_dir["baseurl"] . "/LSD/$tmp_dir/design.png") {
                $output .= "<a class='button' target='_blank' href='" . $upload_dir["baseurl"] . "/LSD/$tmp_dir/design.png' download='" . $upload_dir["basedir"] . "/LSD/$tmp_dir/design.png'>" . esc_html__("Download design", "wpd") . "</a> ";
            }
            if ('yes' === $design_composition_visible) {
                $output .= $this->lsd_get_formatted_data($data);
            }
        }
        //Escaping output.
        echo $output;
    }

    /**
     * Creates a compressed zip file
     * @param type $source Input directory path to zip
     * @param type $destination Output file path
     * @return boolean
     */
    private function generate_design_archive($source, $destination) {

        if (!extension_loaded('zip') || !file_exists($source)) {
            return false;
        }

        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            return false;
        }

        $source = str_replace('\\', DIRECTORY_SEPARATOR, realpath($source));

        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

            foreach ($files as $file) {
                $file = str_replace('\\', DIRECTORY_SEPARATOR, $file);

// Ignore "." and ".." folders
                if (in_array(substr($file, strrpos($file, '/') + 1), array('.', '..'), true)) {
                    continue;
                }

                $file = realpath($file);

                if (is_dir($file) === true) {
                    $zip->addEmptyDir(str_replace($source . DIRECTORY_SEPARATOR, '', $file . DIRECTORY_SEPARATOR));
                } elseif (is_file($file) === true) {
                    $zip->addFromString(str_replace($source . DIRECTORY_SEPARATOR, '', $file), $this->url_get_contents($file));
                }
            }
        } elseif (is_file($source) === true) {
            $zip->addFromString(basename($source), $this->url_get_contents($source));
        }

        return $zip->close();
    }

    /**
     * Replacement just in case file_get_contents fails
     * @param type $Url
     * @return type
     */
    public function url_get_contents($Url) {
//If it's a path, we prefer use the file_get_contents.
        if (function_exists('curl_init') && !file_exists($Url)) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $Url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $output = curl_exec($ch);
            curl_close($ch);
        } else
            $output = file_get_contents($Url);

        return $output;
    }

    /**
     * Function to display user design on account page.
     * @global type $lsd_settings
     * @param type $item_id Item id.
     * @param type $item Cart item object.
     * @param type $order Order object.
     * @return type
     */
    public function lsd_get_user_account_products_meta($item_id, $item, $order) {
        if (!is_account_page()) {
            return;
        }
        global $lsd_settings;
        $options = $lsd_settings['lsd-general-options'];
        $output = '';
        $download_btn = get_proper_value($options, 'lsd-user-account-download-btn', '');
        $invalid_statuses = array('wc-cancelled', 'wc-refunded', 'wc-failed');
        if (!in_array($order->get_status(), $invalid_statuses, true) && '0' !== $download_btn) {
            if ('0' !== $item['variation_id'] && isset($item['variation_id']) && (!empty($item['variation_id']) )) {
                $product = wc_get_product($item['variation_id']);
            } else {
                if (isset($item['product_id'])) {
                    $product = wc_get_product($item['product_id']);
                }
            }

            $item_id = uniqid();
            ob_start();
            $this->lsd_get_order_custom_admin_data($item_id, $item, $product);
            $admin_data = ob_get_contents();
            ob_end_clean();
            $output .= $admin_data;
        }
//Escaping output.
        $allowed_tags = lsd_allowed_tags();
        echo wp_kses($output, $allowed_tags);
    }

    /**
     * Function to handle design integration in mail as attachement.
     * @global type $lsd_settings
     * @param type $attachments Files attached to mail.
     * @param type $status Order status.
     * @param type $order Order object.
     * @return type
     */
    public function lsd_add_order_design_to_mail($attachments, $status, $order) {
        global $lsd_settings;
        $options = $lsd_settings['lsd-general-options'];
        $download_btn = $options['lsd-send-design-mail'];
        if ('0' !== $download_btn) {
            $allowed_statuses = array('new_order', 'customer_invoice', 'customer_processing_order', 'customer_completed_order');
            if (isset($status) && in_array($status, $allowed_statuses, true)) {
                $items = $order->get_items();
                foreach ($items as $order_item_id => $item) {
                    $upload_dir = wp_upload_dir();
                    if (isset($item['lsd_data'])) {
                        $data = $item['lsd_data'];
                        array_push($attachments, $data["output"]["zip"]);
                    }
                }
            }
            $attachments = str_replace('"', '', $attachments);
        }
        return $attachments;
    }

    /**
     * Function to calculate cart item price
     * @param type $cart Woocommerce cart.
     */
    public function lsd_get_cart_item_price($cart) {
        foreach ($cart->cart_contents as $key => $cart_item) {
            if (isset($cart_item['variation_id']) && 0 !== $cart_item['variation_id']) {
                $lsd_product = new LSD_Product($cart_item['variation_id']);
            } else {
                $lsd_product = new LSD_Product($cart_item['product_id']);
            }
            if (isset($lsd_product) && $lsd_product->is_customizable()) {
                $price = (double) $lsd_product->product->get_price();
                if (isset($lsd_product->settings)) {
                    $lsd_metas = $lsd_product->settings;
                    $bg_palette = get_proper_value($lsd_metas, 'bg-colors-palette', '');
                    $text_palette = get_proper_value($lsd_metas, 'text-colors-palette', '');
                    $shadow_palette = get_proper_value($lsd_metas, 'shadow-colors-palette', '');
                    $outline_palette = get_proper_value($lsd_metas, 'outline-colors-palette', '');
                    
                    $pricing_rules = get_proper_value($lsd_metas, 'pricing-rules', '');
                }
                if (isset($cart_item) && isset($cart_item['lsd_generated_data'])) {
                    $lsd_generated_data = $cart_item['lsd_generated_data'];
                }
                if (isset($bg_palette) && !empty($bg_palette) && isset($lsd_generated_data)) {
                    foreach ($bg_palette as $key => $value) {
                        if (isset($value['hex']) && isset($lsd_generated_data['canvas_color']) && strtoupper($value['hex']) === strtoupper($lsd_generated_data['canvas_color']) ) {
                            $price += (double) $value['price'];
                        }
                    }
                }
                if (isset($text_palette) && !empty($text_palette) && isset($lsd_generated_data)) {
                    foreach ($text_palette as $key => $value) {
                        if (isset($value['hex']) && isset($lsd_generated_data['text_color']) && strtoupper($value['hex']) === strtoupper($lsd_generated_data['text_color']) ) {
                            $price += (double) $value['price'];
                        }
                    }
                }
                
                if (isset($shadow_palette) && !empty($shadow_palette) && isset($lsd_generated_data)) {
                    foreach ($shadow_palette as $key => $value) {
                        if (isset($value['hex']) && isset($lsd_generated_data['text_color']) && strtoupper($value['hex']) === strtoupper($lsd_generated_data['shadow_color']) ) {
                            $price += (double) $value['price'];
                        }
                    }
                }
                
                if (isset($outline_palette) && !empty($outline_palette) && isset($lsd_generated_data)) {
                    foreach ($outline_palette as $key => $value) {
                        if (isset($value['hex']) && isset($lsd_generated_data['text_color']) && strtoupper($value['hex']) === strtoupper($lsd_generated_data['outline_color']) ) {
                            $price += (double) $value['price'];
                        }
                    }
                }
                
                $additionnal_price_per_item = 0;
                $additionnal_price_on_design = 0;

                if (isset($pricing_rules) && !empty($pricing_rules) && isset($lsd_generated_data)) {
                    foreach ($pricing_rules as $key => $rules) {
                        $verify_result = $this->lsd_condition_verify($rules, $lsd_generated_data);
                        if ($verify_result) {
                            if ($verify_result['verify']) {
                                $additionnal_price_per_item = $additionnal_price_per_item + (int) $verify_result['additionnal_price'];
                                if (isset($rules['a_price'])) {
                                    if ($rules['scope'] === 'design' || $verify_result['param'] === 'if_selected_font') {
                                        $additionnal_price_on_design = $additionnal_price_on_design + (double) $rules['a_price'];
                                    }
                                }
                            }
                        }
                    }
                }
                $price += $additionnal_price_per_item + $additionnal_price_on_design;
                $cart_item['data']->set_price($price);
            }
        }
    }

    /**
     * Function that verify rules conditions
     * @param type $rules Array of pricing rules.
     * @param type $lsd_generated_data Generatedd data.
     * @return boolean
     */
    public function lsd_condition_verify($rules, $lsd_generated_data) {
        $comparaison_result = [];
        if (isset($rules) && !empty($rules)) {
            if (isset($rules['rules']) && !empty($rules['rules'])) {
                foreach ($rules['rules'] as $key => $rule) {
                    switch ($rule['param']) {
                        case 'txt_nb_chars':
                            if (isset($lsd_generated_data) && isset($lsd_generated_data['text'])) {
                                $text_size = sizeof(str_split($lsd_generated_data['text']));
                                $result = $this->lsd_comparaison($rule['operator'], $text_size, $rule['value']);
                                $result['param'] = $rule['param'];
                                $comparaison_result[] = $result;
                            }
                            break;
                        case 'txt_nb_lines':
                            if (isset($lsd_generated_data) && isset($lsd_generated_data['text'])) {
                                $nb_line = sizeof(explode('<br/>', $lsd_generated_data['text']));
                                $result = $this->lsd_comparaison($rule['operator'], $nb_line, $rule['value']);
                                $result['param'] = $rule['param'];
                                $comparaison_result[] = $result;
                            }
                            break;
                        case 'if_canvas_square_unit':
                            if (isset($lsd_generated_data) && isset($lsd_generated_data['custom-width']) && isset($lsd_generated_data['custom-height'])) {
                                $canvas_square = (int) $lsd_generated_data['custom-width'] * (int) $lsd_generated_data['custom-height'];
                                $result = $this->lsd_comparaison($rule['operator'], $canvas_square, $rule['value']);
                                $result['param'] = $rule['param'];
                                $comparaison_result[] = $result;
                            }
                            break;
                        case 'if_selected_font':
                            if (isset($lsd_generated_data) && isset($lsd_generated_data['font'])) {
                                $result = $this->lsd_comparaison($rule['operator'], $lsd_generated_data['font'], $rule['value']);
                                $result['param'] = $rule['param'];
                                $comparaison_result[] = $result;
                            }
                            break;
                        case 'if_char_spacing':
                            if (isset($lsd_generated_data) && isset($lsd_generated_data['spacing'])) {
                                $result = $this->lsd_comparaison($rule['operator'], $lsd_generated_data['spacing'], $rule['value']);
                                $result['param'] = $rule['param'];
                                $comparaison_result[] = $result;
                            }
                            break;
                        default:
                            $comparaison_result = apply_filters('lsd_custom_palette_tpl', $comparaison_result, $rules, $lsd_generated_data);
                            break;
                    }
                }

                $verify = 0;
                foreach ($comparaison_result as $key => $value) {
                    if ($value['result']) {
                        $verify++;
                    }
                }

                $additionnal_price_per_item = 0;
                $result = array('verify' => false);

                if (sizeof($comparaison_result) === $verify && isset($value) && !empty($value)) {
                    $result['verify'] = true;
                    $result['param'] = $value['param'];
                    foreach ($comparaison_result as $key => $value) {
                        if ($rules['scope'] === 'item' && isset($value['remaining_number'])) {
                            $additionnal_price_per_item = $additionnal_price_per_item + ($value['remaining_number'] * $rules['a_price']);
                        } else {
                            $additionnal_price_per_item = $additionnal_price_per_item + 0;
                        }
                    }
                    $result['additionnal_price'] = $additionnal_price_per_item;
                }
                return $result;
            }
        }
    }

    /**
     * Function that compares values
     * @param type $operator Given operator.
     * @param type $given_value Given value.
     * @param type $rule_value Rule value.
     * @return type
     */
    public function lsd_comparaison($operator, $given_value, $rule_value) {
        $data = [];
        if ('>' === $operator) {
            $data['result'] = $given_value > $rule_value;
            if ($given_value > $rule_value) {
                $data['remaining_number'] = $given_value - $rule_value;
            }
            return $data;
        } elseif ('>=' === $operator) {
            $data['result'] = $given_value >= $rule_value;
            if ($given_value >= $rule_value) {
                $data['remaining_number'] = ($given_value - $rule_value) + 1;
            }
            return $data;
        } elseif ('<' === $operator) {
            $data['result'] = $given_value < $rule_value;
            if ($given_value < $rule_value) {
                $data['remaining_number'] = $rule_value - $given_value;
            }
            return $data;
        } elseif ('<=' === $operator) {
            $data['result'] = $given_value <= $rule_value;
            if ($given_value < $rule_value) {
                $data['remaining_number'] = ($rule_value - $given_value) + 1;
            }
            return $data;
        } elseif ('==' === $operator) {
            $data['result'] = $given_value == $rule_value;
            if (is_numeric($given_value) && is_numeric($rule_value) && $given_value === $rule_value) {
                $data['remaining_number'] = $given_value;
            }
            return $data;
        } else {
            $data['result'] = $given_value != $rule_value;
            if (is_numeric($given_value) && is_numeric($rule_value) && $given_value !== $rule_value) {
                $data['remaining_number'] = $given_value;
            }
            return $data;
        }
    }

    /**
     * Save the file as pdf
     * @param type $variation_id
     * @param type $input_file
     * @param type $output_file
     * @param type $pdf_fonts
     */
    private function lsd_save_pdf_output($output_file, $data) {
        $product_id = $data['product_id'];
        $config = lsd_get_product_config($product_id);
        $product_metas = $config->settings;
        $product_output_settings = get_proper_value($product_metas, "output-settings", array());
        $text = $data['text'];

        $font_ttf = $data['font-ttf'];

        if ($data['style'] === 'bold') {
            $fonturl = $data['font-url'] . ":700";
        } elseif ($data['style'] === 'italic') {
            $fonturl = $data['font-url'] . ":400i";
        } elseif ($data['style'] === 'both') {
            $fonturl = $data['font-url'] . ":700i";
        } else {
            $fonturl = $data['font-url'];
        }

        if ('' !== $fonturl && '' === $font_ttf) {
            $font_ttf = lsd_retrieve_ttf_from_url($fonturl);
        }
        $pdf_format = get_proper_value($product_output_settings, "pdf-format", "A0");
        $pdf_unit = "mm";
        
        /*
         * We convert font size in point and all other values to mm
         */
        if($data['unit'] !== "mm"){
            if ($data['unit'] !== 'pt'){
                $font_size_in_pixels = lsd_convert_unit_to_px($data['font-size'], $data['unit']);
                $font_size = lsd_convert_px_to_unit($font_size_in_pixels, "pt");
            }else {
                $font_size = $data['font-size'];
            }
            
            if ($pdf_format == "custom") {
                $pdf_width = lsd_convert_px_to_unit( lsd_convert_unit_to_px( get_proper_value($product_output_settings, "pdf-width", 0) , $data['unit']), "mm");
                $pdf_height = lsd_convert_px_to_unit( lsd_convert_unit_to_px( get_proper_value($product_output_settings, "pdf-height", 0) , $data['unit']), "mm");
            } else {
                $dimensions = TCPDF_STATIC::getPageSizeFromFormat($product_output_settings['pdf-format']);
                $pdf_width = lsd_convert_px_to_unit( lsd_convert_unit_to_px( $dimensions[0], $data['unit']), "mm" );
                $pdf_height = lsd_convert_px_to_unit(  lsd_convert_unit_to_px( $dimensions[1], $data['unit']), "mm");
            }
            
            
            $custom_width = lsd_convert_px_to_unit( lsd_convert_unit_to_px( $data['custom-width'],$data['unit']), "mm");
            $custom_height= lsd_convert_px_to_unit( lsd_convert_unit_to_px( $data['custom-height'],$data['unit']), "mm");
            $pdf_margin_lr =  lsd_convert_px_to_unit(  lsd_convert_unit_to_px( get_proper_value($product_output_settings, "pdf-margin-lr", 0), $data['unit'] ), "mm" );
            $pdf_margin_tb =  lsd_convert_px_to_unit(  lsd_convert_unit_to_px( get_proper_value($product_output_settings, "pdf-margin-tb", 0), $data['unit'] ), "mm" );
            
        }else{
            $custom_width = $data['custom-width'];
            $custom_height= $data['custom-height'];
            $font_size_in_pixels = lsd_convert_unit_to_px($data['font-size'], $data['unit']);
            $font_size = lsd_convert_px_to_unit($font_size_in_pixels, "pt");
            if ($pdf_format == "custom") {
                $pdf_width = get_proper_value($product_output_settings, "pdf-width", 0);
                $pdf_height = get_proper_value($product_output_settings, "pdf-height", 0);
            } else {
                $dimensions = TCPDF_STATIC::getPageSizeFromFormat($product_output_settings['pdf-format']);
                $pdf_width = $dimensions[0];
                $pdf_height =$dimensions[1];
            }
            $pdf_margin_lr = get_proper_value($product_output_settings, "pdf-margin-lr", 0);
            $pdf_margin_tb = get_proper_value($product_output_settings, "pdf-margin-tb", 0);
        }
        
        
        $shadow = get_proper_value($data, "shadow", 0);
        $outline = get_proper_value($data, "outline", 0);

        
        if (strpos($pdf_format, ",") !== FALSE) {
            $formats = explode(",", $pdf_format);
            $pdf_format = $formats[0];
        }

        if ($pdf_width < $custom_width) {
            $pdf_width = $custom_width;
        }

        if ($pdf_height < $custom_height) {
            $pdf_height = $custom_height;
        }

        $output_format = array($pdf_width, $pdf_height);

//set pdf orientation
        $pdf_orientation = get_proper_value($product_output_settings, "pdf-orientation", "P");

        $pdf = new TCPDF($pdf_orientation, $pdf_unit, $output_format, true, 'UTF-8', false);

//set pdf unit
        $pdf->setPageUnit($pdf_unit);
        $pdf->setPageOrientation($pdf_orientation);
        $pdf->SetCreator("Lettering and signs Designer by ORION");
        $pdf->SetAuthor('Lettering and signs Designer by ORION');
        $pdf->SetTitle('PDF output');
        $pdf->setPrintHeader(false);
        $pdf->setPrintFooter(false);

        //set margins
        $pdf->SetMargins($pdf_margin_lr, $pdf_margin_tb, -1, true);
        $pdf->SetHeaderMargin($pdf_margin_tb);
        $pdf->SetFooterMargin($pdf_margin_tb);


// set auto page breaks
//$pdf->SetAutoPageBreak(TRUE, $pdf_margin_tb);
// convert TTF font to TCPDF format and store it on the fonts folder
        if ($font_ttf === '') {
// set default monospaced font
            $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
        } else {
            $fontname = $pdf->addTTFfont($font_ttf, 'TrueTypeUnicode', '', 96);
            $pdf->SetFont($fontname, '', $font_size, '', 'default', false);
        }

        $text_col = lsd_hex_to_rgb($data['text_color']);

        $pdf->SetTextColor($text_col['0'], $text_col['1'], $text_col['2']);

        $pdf->AddPage();

        if ($shadow > 0) {
            $shadow_color = lsd_hex_to_rgb($data['shadow_color']);
// set text shadow effect
            $pdf->setTextShadow(array('enabled' => true, 'depth_w' => $shadow, 'depth_h' => $data['shadow'], 'color' => $shadow_color, 'opacity' => 1, 'blend_mode' => 'Normal'));
        }

        if ($outline > 0) {
            $outline_color = lsd_hex_to_rgb($data['outline_color']);
// = $outline_color;
            $pdf->SetDrawColor($outline_color['0'], $outline_color['1'], $outline_color['2']);
        }



//Write text
        $pdf->Text(0, 0, $text, $outline, false, true, 0, 0, '', false, '', 0, false, 'T', 'T', false);


        $pdf->Output($output_file, 'F');
    }

    /**
     * Execute imagick command.
     * @param type $cmd The actual command to be executed
     * @return array
     */
    private function lsd_exec($cmd) {
        $output = array();
        exec("$cmd 2>&1", $output);
        return $output;
    }

    /**
     * Export data to archive
     * @param string $generation_dir Working directory path
     * @param array data Data to export
     * @param int variation_id Product/Variation ID
     * @return boolean|string
     */
    private function export_data_to_files($generation_dir, $generation_url, $data, $zip_name) {
        global $lsd_settings;
        $global_output_settings = $lsd_settings['lsd-output-options'];
//Set this value to true to have the pdf generated.
        $generate_pdf = false;
        $generate_zip = false;
        $wpc_cmyk_conversion = "yes";

        $output_arr = array();

        $product_id = $data['product_id'];
        $config = lsd_get_product_config($product_id);
        $product_metas = $config->settings;
        $product_output_settings = get_proper_value($product_metas, "output-settings", array());

        if ($product_output_settings['output-format'] == 'pdf+png' || $product_output_settings['output-format'] == 'pdf+jpg' || $product_output_settings['output-format'] == 'pdf+svg') {
            $generate_pdf = true;
            $format = explode("+", $product_output_settings['output-format']);
            $lsd_img_format = $format[1];
        } else {
            $lsd_img_format = $product_output_settings['output-format'];
        }

        if ($product_output_settings['zip-output'] == 'yes') {
            $generate_zip = true;
        }

        $lsd_cmyk_conversion = get_proper_value($product_output_settings, "lsd-cmyk-conversion", "no");
        if (!class_exists("Imagick")) {
            $lsd_cmyk_conversion = "no";
        }

        $data['format'] = $lsd_img_format;

        $allowed_extensions = array('png', 'jpg');
        if (!in_array($lsd_img_format, $allowed_extensions)) {
            return false;
        }


        if (!wp_mkdir_p($generation_dir)) {
            echo "Can't create part directory...";
            die();
        }

        if (!isset($data['font']) || '' === $data['font']) {
            echo 'No font selected. Make sure you have defined some fonts to be used. ';
            die();
        }

        if (!isset($data['canvas_color'])) {
            $data['canvas_color'] = '';
        }

        if (!isset($data['shadow_color'])) {
            $data['shadow_color'] = "rgb(132,132,132)";
        }

        if (!isset($data['outline_color'])) {
            $data['outline_color'] = "rgb(132,132,132)";
        }

        if (!isset($data['font-url'])) {
            $data['font-url'] = "";
        }

        if (!isset($data['font-ttf'])) {
            $data['font-ttf'] = "";
        }

        if (!isset($data['alignment'])) {
            $data['alignment'] = 'left';
        }

        if (!isset($data['spacing'])) {
            $data['spacing'] = 1;
        }

        if (!isset($data['shadow'])) {
            $data['shadow'] = 0;
        }

        if (!isset($data['outline'])) {
            $data['outline'] = 0;
        }

        if (!isset($data['dimensions-type'])) {
            $data['dimensions-type'] = 'canvas-dimensions';
        }

        if (!isset($data['font-size'])) {
            $data['font-size'] = 0;
        }

        $text = $data['text'];
        if ('' === $text) {
            $text = 'Your text';
        }

        $data['text'] = $text;

        if (!class_exists("Imagick")) {
            $wpc_cmyk_conversion = "no";
        }

        if ("yes" === $wpc_cmyk_conversion) {
            $wpc_cmyk_profil = get_proper_value($global_output_settings, "lsd-cmyk-profil", false);
            if ($wpc_cmyk_profil) {
                $wpc_cmyk_profil = get_home_path() . $wpc_cmyk_profil;
                $cmd = "convert $generation_dir -colorspace cmyk -profile " . $wpc_cmyk_profil . " $generation_dir";
            } else {
                $cmd = "convert $generation_dir -colorspace cmyk $generation_dir";
            }

            $exec_result = $this->lsd_exec($cmd);
            if (!empty($exec_result)) {
                $output_arr["cmyk-error"] = $exec_result[0];
            }
        }

//generate png images
        $data['path'] = $generation_dir . '/design.' . $data['format'];
        $data['url'] = $generation_url . '/design.' . $data['format'];
        $full_image = $this->lsd_generate_image($data);

//generating resized image
        $resized_data = $data;
        $resized_data['path'] = $generation_dir . '/resized-design.' . $data['format'];
        $resized_data['url'] = $generation_url . '/resized-design.' . $data['format'];
        $resized_data['custom-width'] = intval($data['custom-width']) / 2;
        $resized_data['custom-height'] = intval($data['custom-height']) / 2;
        $resized_data['font-size'] = intval($data['font-size']) / 2;
        $resized_image = $this->lsd_generate_image($resized_data);

//Part pdf
        if ($generate_pdf) {
            $output_pdf_file_path = $generation_dir . "/design.pdf";
            $pdf_generation_method = apply_filters("lsd_pdf_generation_method", array($this, "lsd_save_pdf_output"));

//Generation de pdf.
            call_user_func($pdf_generation_method, $output_pdf_file_path, $data);
            $output_arr["file"] = "design.pdf";
        }

        $output_arr['full'] = $full_image;
        $output_arr['thumb'] = $resized_image;

        if ($generate_zip) {
            $this->generate_design_archive($generation_dir, "$generation_dir/$zip_name");
            $output_arr['zip'] = $generation_dir / $zip_name;
        }


        return $output_arr;
    }

}
